@@ -52,6 +52,7 @@ static void gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_d
 static void gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data);
 static gboolean rs_gtk_tree_model_count_helper(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 static gint rs_gtk_tree_model_count(GtkTreeModel *model);
+static inline guint8 convert_color_channel (guint8 src, guint8 alpha);
 
 static const gchar *color_profiles[] = {
 	"*.icc", 
@@ -373,7 +374,7 @@ cms_enable_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	rs_conf_set_boolean(CONF_CMS_ENABLED, togglebutton->active);
 	rs_cms_enable(rs->cms, togglebutton->active);
-	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, PROFILE_DISPLAY));
+	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
 	return;
 }
 
@@ -383,10 +384,10 @@ gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	gchar *filename;
 	rs_conf_set_integer(CONF_CMS_IN_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
-	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_IN);
-	rs_cms_set_profile(rs->cms, PROFILE_INPUT, filename);
+	filename = rs_conf_get_cms_profile(CMS_PROFILE_INPUT);
+	rs_cms_set_profile(rs->cms, CMS_PROFILE_INPUT, filename);
 	g_free(filename);
-	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview), FALSE);
+	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
 	return;
 }
 
@@ -397,10 +398,10 @@ gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	gchar *filename;
 
 	rs_conf_set_integer(CONF_CMS_DI_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
-	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_DISPLAY);
-	rs_cms_set_profile(rs->cms, PROFILE_DISPLAY, filename);
+	filename = rs_conf_get_cms_profile(CMS_PROFILE_DISPLAY);
+	rs_cms_set_profile(rs->cms, CMS_PROFILE_DISPLAY, filename);
 	g_free(filename);
-	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview), FALSE);
+	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
 	return;
 }
 
@@ -411,8 +412,8 @@ gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	gchar *filename;
 
 	rs_conf_set_integer(CONF_CMS_EX_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
-	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_EXPORT);
-	rs_cms_set_profile(rs->cms, PROFILE_EXPORT, filename);
+	filename = rs_conf_get_cms_profile(CMS_PROFILE_EXPORT);
+	rs_cms_set_profile(rs->cms, CMS_PROFILE_EXPORT, filename);
 	g_free(filename);
 	return;
 }
@@ -429,7 +430,7 @@ gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 }
 
 static gchar *
-gui_cms_choose_profile(const gchar *path)
+gui_cms_choose_profile(const gchar *path, const CMS_PROFILE profile)
 {
 	gchar *ret = NULL;
 	GtkWidget *fc;
@@ -477,7 +478,7 @@ gui_cms_choose_profile(const gchar *path)
 		gchar *filename;
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
 
-		if (rs_cms_is_profile_valid(filename))
+		if (rs_cms_is_profile_valid(filename, profile))
 			ret = filename;
 		else
 		{
@@ -504,7 +505,7 @@ gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data)
 	GtkWidget *combobox = GTK_WIDGET(user_data);
 	gchar *filename;
 
-	filename = gui_cms_choose_profile(NULL);
+	filename = gui_cms_choose_profile(NULL, CMS_PROFILE_INPUT);
 
 	if (filename)
 	{
@@ -523,7 +524,7 @@ gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data)
 	GtkWidget *combobox = GTK_WIDGET(user_data);
 	gchar *filename; 
 
-	filename = gui_cms_choose_profile(NULL);
+	filename = gui_cms_choose_profile(NULL, CMS_PROFILE_DISPLAY);
 
 	if (filename)
 	{
@@ -542,7 +543,7 @@ gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data)
 	GtkWidget *combobox = GTK_WIDGET(user_data);
 	gchar *filename;
 
-	filename = gui_cms_choose_profile(NULL);
+	filename = gui_cms_choose_profile(NULL, CMS_PROFILE_EXPORT);
 
 	if (filename)
 	{
@@ -606,11 +607,25 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 		temp_conf_string = (gchar *) temp_conf_gslist->data;
 		if (g_file_test(temp_conf_string, G_FILE_TEST_EXISTS))
 		{
-			cmsHPROFILE color_profile = cmsOpenProfileFromFile(temp_conf_string, "r");
-			if (color_profile)
+			gchar *name, *desc;
+			if (cms_get_profile_info_from_file(temp_conf_string, &name, NULL, &desc))
 			{
-				cmsCloseProfile(color_profile);	
-				gtk_combo_box_append_text(GTK_COMBO_BOX(cms_in_profile_combobox), g_basename(temp_conf_string));
+				if (name)
+				{
+					GString *gs = g_string_new(name);
+					if (!g_str_equal(name, desc))
+					{
+						gs = g_string_append(gs, "\n ");
+						gs = g_string_append(gs, desc);
+						g_free(desc);
+					}
+					gtk_combo_box_append_text(GTK_COMBO_BOX(cms_in_profile_combobox), gs->str);
+					g_free(name);
+					g_string_free(gs, TRUE);
+				}
+				else
+					gtk_combo_box_append_text(GTK_COMBO_BOX(cms_in_profile_combobox), g_basename(temp_conf_string));
+
 				temp_new_gslist = g_slist_append(temp_new_gslist, (gpointer *) temp_conf_string);
 			}
 		}
@@ -645,11 +660,25 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 		temp_conf_string = (gchar *) temp_conf_gslist->data;
 		if (g_file_test(temp_conf_string, G_FILE_TEST_EXISTS))
 		{
-			cmsHPROFILE color_profile = cmsOpenProfileFromFile(temp_conf_string, "r");
-			if (color_profile)
+			gchar *name, *desc;
+			if (cms_get_profile_info_from_file(temp_conf_string, &name, NULL, &desc))
 			{
-				cmsCloseProfile(color_profile);	
-				gtk_combo_box_append_text(GTK_COMBO_BOX(cms_di_profile_combobox), g_basename(temp_conf_string));
+				if (name)
+				{
+					GString *gs = g_string_new(name);
+					if (!g_str_equal(name, desc))
+					{
+						gs = g_string_append(gs, "\n ");
+						gs = g_string_append(gs, desc);
+						g_free(desc);
+					}
+					gtk_combo_box_append_text(GTK_COMBO_BOX(cms_di_profile_combobox), gs->str);
+					g_free(name);
+					g_string_free(gs, TRUE);
+				}
+				else
+					gtk_combo_box_append_text(GTK_COMBO_BOX(cms_di_profile_combobox), g_basename(temp_conf_string));
+
 				temp_new_gslist = g_slist_append(temp_new_gslist, (gpointer *) temp_conf_string);
 			}
 		}
@@ -684,11 +713,25 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 		temp_conf_string = (gchar *) temp_conf_gslist->data;
 		if (g_file_test(temp_conf_string, G_FILE_TEST_EXISTS))
 		{
-			cmsHPROFILE color_profile = cmsOpenProfileFromFile(temp_conf_string, "r");
-			if (color_profile)
+			gchar *name, *desc;
+			if (cms_get_profile_info_from_file(temp_conf_string, &name, NULL, &desc))
 			{
-				cmsCloseProfile(color_profile);	
-				gtk_combo_box_append_text(GTK_COMBO_BOX(cms_ex_profile_combobox), g_basename(temp_conf_string));
+				if (name)
+				{
+					GString *gs = g_string_new(name);
+					if (!g_str_equal(name, desc))
+					{
+						gs = g_string_append(gs, "\n ");
+						gs = g_string_append(gs, desc);
+						g_free(desc);
+					}
+					gtk_combo_box_append_text(GTK_COMBO_BOX(cms_ex_profile_combobox), gs->str);
+					g_free(name);
+					g_string_free(gs, TRUE);
+				}
+				else
+					gtk_combo_box_append_text(GTK_COMBO_BOX(cms_ex_profile_combobox), g_basename(temp_conf_string));
+
 				temp_new_gslist = g_slist_append(temp_new_gslist, (gpointer *) temp_conf_string);
 			}
 		}
@@ -928,3 +971,148 @@ gui_select_theme(RS_THEME theme)
 
 	g_static_mutex_unlock(&lock);
 }
+
+/**
+ * http://davyd.ucc.asn.au/projects/gtk-utils/cairo_convert_to_pixbuf.html 
+ */
+static inline guint8
+convert_color_channel (guint8 src, guint8 alpha)
+{
+	return alpha ? ((src << 8) - src) / alpha : 0;
+}
+
+/**
+ * cairo_convert_to_pixbuf:
+ * Converts from a Cairo image surface to a GdkPixbuf. Why does GTK+ not
+ * implement this?
+ * http://davyd.ucc.asn.au/projects/gtk-utils/cairo_convert_to_pixbuf.html 
+ */
+GdkPixbuf *
+cairo_convert_to_pixbuf (cairo_surface_t *surface)
+{
+	GdkPixbuf *pixbuf;
+	int width, height;
+	int srcstride, dststride;
+	guchar *srcpixels, *dstpixels;
+	guchar *srcpixel, *dstpixel;
+	int n_channels;
+	int x, y;
+
+	switch (cairo_image_surface_get_format (surface))
+	{
+		case CAIRO_FORMAT_ARGB32:
+		case CAIRO_FORMAT_RGB24:
+			break;
+
+		default:
+			g_critical ("This Cairo surface format not supported");
+			return NULL;
+			break;
+	}
+
+	width = cairo_image_surface_get_width (surface);
+	height = cairo_image_surface_get_height (surface);
+	srcstride = cairo_image_surface_get_stride (surface);
+	srcpixels = cairo_image_surface_get_data (surface);
+
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
+			width, height);
+	dststride = gdk_pixbuf_get_rowstride (pixbuf);
+	dstpixels = gdk_pixbuf_get_pixels (pixbuf);
+	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+
+	for (y = 0; y < height; y++)
+	{
+		for (x = 0; x < width; x++)
+		{
+			srcpixel = srcpixels + y * srcstride + x * 4;
+			dstpixel = dstpixels + y * dststride + x * n_channels;
+
+			dstpixel[0] = convert_color_channel (srcpixel[2],
+							     srcpixel[3]);
+			dstpixel[1] = convert_color_channel (srcpixel[1],
+							     srcpixel[3]);
+			dstpixel[2] = convert_color_channel (srcpixel[0],
+							     srcpixel[3]);
+			dstpixel[3] = srcpixel[3];
+		}
+	}
+
+	return pixbuf;
+}
+
+GtkWidget *
+gui_dialog_make_from_text(const gchar *stock_id, gchar *primary_text, gchar *secondary_text)
+{
+	GtkWidget *secondary_label;
+
+	secondary_label = gtk_label_new (NULL);
+	gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
+	gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0.0, 0.5);
+	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
+	gtk_label_set_markup (GTK_LABEL (secondary_label), secondary_text);
+	
+	return(gui_dialog_make_from_widget(stock_id, primary_text, secondary_label));
+}
+
+GtkWidget *
+gui_dialog_make_from_widget(const gchar *stock_id, gchar *primary_text, GtkWidget *widget)
+{
+	GtkWidget *dialog, *image, *hhbox, *vvbox;
+	GtkWidget *primary_label;
+	gchar *str;
+
+	image = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_DIALOG);
+	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+	dialog = gtk_dialog_new();
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	gtk_window_set_title (GTK_WINDOW (dialog), "");
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
+
+	primary_label = gtk_label_new (NULL);
+	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
+	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (primary_label), 0.0, 0.5);
+	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
+	str = g_strconcat("<span weight=\"bold\" size=\"larger\">", primary_text, "</span>", NULL);
+	gtk_label_set_markup (GTK_LABEL (primary_label), str);
+	g_free(str);
+
+	hhbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hhbox), 5);
+	gtk_box_pack_start (GTK_BOX (hhbox), image, FALSE, FALSE, 0);
+	vvbox = gtk_vbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (hhbox), vvbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vvbox), primary_label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vvbox), widget, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hhbox, FALSE, FALSE, 0);
+
+	return(dialog);
+}
+
+/**
+ * Creates a new GtkButton widget.
+ * @param stock_id A stock id registered with GTK+
+ * @param label The text to show besides the icon
+ * @return a new GtkButton
+ */
+GtkWidget *
+gui_button_new_from_stock_with_label(const gchar *stock_id, const gchar *label)
+{
+	GtkWidget *button;
+	GtkWidget *stock;
+
+	g_assert(stock_id);
+	g_assert(label);
+
+	stock = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_BUTTON);
+	button = gtk_button_new_with_label(label);
+	gtk_button_set_image(GTK_BUTTON(button), stock);
+
+	return button;
+}
