@@ -42,6 +42,7 @@ extern gboolean window_key_press_event(GtkWidget *widget, GdkEventKey *event);
 extern void pos_menu_below_widget(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data);
 extern GtkWidget *gui_framed(GtkWidget *widget, const gchar *title, GtkShadowType shadowtype);
 extern GtkWidget *gui_aligned(GtkWidget *widget, const gfloat xalign, const gfloat yalign, const gfloat xscale, const gfloat yscale);
+extern GdkPixbuf *cairo_convert_to_pixbuf (cairo_surface_t *surface);
 
 /**
  * Build and show a popup-menu
@@ -58,3 +59,14 @@ typedef enum {
 } RS_THEME;
 
 extern void gui_select_theme(RS_THEME theme);
+
+extern GtkWidget *gui_dialog_make_from_text(const gchar *stock_id, gchar *primary_text, gchar *secondary_text);
+extern GtkWidget *gui_dialog_make_from_widget(const gchar *stock_id, gchar *primary_text, GtkWidget *widget);
+
+/**
+ * Creates a new GtkButton widget.
+ * @param stock_id A stock id registered with GTK+
+ * @param label The text to show besides the icon
+ * @return a new GtkButton
+ */
+extern GtkWidget *gui_button_new_from_stock_with_label(const gchar *stock_id, const gchar *label);
