@@ -58,6 +58,7 @@ static GtkWidget *hbox;
 GdkGC *dashed;
 GdkGC *grid;
 
+static gboolean open_photo(RS_BLOB *rs, const gchar *filename);
 static void gui_preview_bg_color_changed(GtkColorButton *widget, RS_BLOB *rs);
 static gboolean gui_fullscreen_iconbox_callback(GtkWidget *widget, GdkEventWindowState *event, GtkWidget *iconbox);
 static gboolean gui_fullscreen_toolbox_callback(GtkWidget *widget, GdkEventWindowState *event, GtkWidget *toolbox);
@@ -150,14 +151,39 @@ update_preview_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs)
 	return(FALSE);
 }
 
+static gboolean
+open_photo(RS_BLOB *rs, const gchar *filename)
+{
+	RS_PHOTO *photo;
+	RS_FILETYPE *filetype;
+	extern GtkLabel *infolabel;
+	gchar *label;
+
+	if ((filetype = rs_filetype_get(filename, TRUE)))
+	{
+		rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), NULL);
+		photo = rs_photo_load_from_file(filename, FALSE);
+
+		if (photo)
+			rs_photo_close(rs->photo);
+		else
+		{
+			gui_set_busy(FALSE);
+			return FALSE;
+		}
+
+		label = rs_metadata_get_short_description(photo->metadata);
+		gtk_label_set_text(infolabel, label);
+		g_free(label);
+
+		rs_set_photo(rs, photo);
+	}
+	return TRUE;
+}
+
 static void
 icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 {
-	RS_FILETYPE *filetype;
-	RS_PHOTO *photo;
-	extern GtkLabel *infolabel;
-	GString *label;
-	gboolean cache_loaded;
 	guint msgid;
 
 	if (name!=NULL)
@@ -178,85 +204,32 @@ icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 			g_list_free(selected);
 		}
 
-		if ((filetype = rs_filetype_get(name, TRUE)))
+		if (!open_photo(rs, name))
 		{
-			rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), NULL);
-			photo = rs_get_preloaded(name);
-			if (!photo)
-				photo = filetype->load(name, FALSE);
-			if (photo)
-			{
-				rs_photo_close(rs->photo);
-			}
-			else
-			{
-				gui_status_pop(msgid);
-				gui_status_notify(_("Couldn't open photo"));
-				gui_set_busy(FALSE);
-				return;
-			}
-
-			if (filetype->load_meta)
-			{
-				filetype->load_meta(name, photo->metadata);
-				switch (photo->metadata->orientation)
-				{
-					case 90: ORIENTATION_90(photo->orientation);
-						break;
-					case 180: ORIENTATION_180(photo->orientation);
-						break;
-					case 270: ORIENTATION_270(photo->orientation);
-						break;
-				}
-				label = g_string_new("");
-				if (photo->metadata->focallength>0)
-					g_string_append_printf(label, _("%dmm "), photo->metadata->focallength);
-				if (photo->metadata->shutterspeed > 0.0 && photo->metadata->shutterspeed < 4) 
-					g_string_append_printf(label, _("%.1fs "), 1/photo->metadata->shutterspeed);
-				else if (photo->metadata->shutterspeed >= 4)
-					g_string_append_printf(label, _("1/%.0fs "), photo->metadata->shutterspeed);
-				if (photo->metadata->aperture!=0.0)
-					g_string_append_printf(label, _("F/%.1f "), photo->metadata->aperture);
-				if (photo->metadata->iso!=0)
-					g_string_append_printf(label, _("ISO%d"), photo->metadata->iso);
-				gtk_label_set_text(infolabel, label->str);
-				g_string_free(label, TRUE);
-			} else
-				gtk_label_set_text(infolabel, _("No metadata"));
-
-			cache_loaded = rs_cache_load(photo);
-
-			if (!cache_loaded)
-			{
-				gint c;
-				for (c=0;c<3;c++)
-				{
-					/* White balance */
-					if (!rs_photo_set_wb_from_camera(photo, c))
-						rs_photo_set_wb_auto(photo, c);
-
-					/* Contrast */
-					if (photo->metadata->contrast != -1.0)
-						rs_photo_set_contrast(photo, c, photo->metadata->contrast);
-
-					/* Saturation */
-					if (photo->metadata->saturation != -1.0)
-						rs_photo_set_saturation(photo, c, photo->metadata->saturation);
-				}
-			}
-			rs_set_photo(rs, photo);
+			gui_status_pop(msgid);
+			gui_status_notify(_("Couldn't open photo"));
+		}
+		else
+		{
+			gui_status_pop(msgid);
+			gui_status_notify(_("Image opened"));
+			window_title = g_string_new(_("Rawstudio"));
+			g_string_append(window_title, " - ");
+			g_string_append(window_title, rs->photo->filename);
+			gtk_window_set_title(GTK_WINDOW(rawstudio_window), window_title->str);
+			g_string_free(window_title, TRUE);
 		}
-		gui_status_pop(msgid);
-		gui_status_notify(_("Image opened"));
-		window_title = g_string_new(_("Rawstudio"));
-		g_string_append(window_title, " - ");
-		g_string_append(window_title, rs->photo->filename);
-		gtk_window_set_title(GTK_WINDOW(rawstudio_window), window_title->str);
-		g_string_free(window_title, TRUE);
 	}
 	gui_set_busy(FALSE);
 }
 
+static void
+group_activated(gpointer instance, GList *names, RS_BLOB *rs)
+{
+	gchar *filename = (gchar *) g_list_nth_data(names, 0);
+	icon_activated(instance, filename, rs);
+}
+
 static void
 gui_preview_bg_color_changed(GtkColorButton *widget, RS_BLOB *rs)
 {
@@ -520,7 +493,7 @@ gui_make_preference_window(RS_BLOB *rs)
 	gtk_box_pack_start (GTK_BOX (histsize_hbox), histsize, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (preview_page), histsize_hbox, FALSE, TRUE, 0);
 
-	show_filenames = checkbox_from_conf(CONF_SHOW_FILENAMES, _("Show filenames in iconview"), TRUE);
+	show_filenames = checkbox_from_conf(CONF_SHOW_FILENAMES, _("Show filenames in iconview"), DEFAULT_CONF_SHOW_FILENAMES);
 	gtk_box_pack_start (GTK_BOX (preview_page), show_filenames, FALSE, TRUE, 0);
 	g_signal_connect ((gpointer) show_filenames, "toggled",
 		G_CALLBACK (gui_preference_iconview_show_filenames_changed), rs);
@@ -803,60 +776,6 @@ gui_window_make(RS_BLOB *rs)
 	return(GTK_WIDGET(rawstudio_window));
 }
 
-GtkWidget *
-gui_dialog_make_from_text(const gchar *stock_id, gchar *primary_text, gchar *secondary_text)
-{
-	GtkWidget *secondary_label;
-
-	secondary_label = gtk_label_new (NULL);
-	gtk_label_set_line_wrap (GTK_LABEL (secondary_label), TRUE);
-	gtk_label_set_use_markup (GTK_LABEL (secondary_label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (secondary_label), 0.0, 0.5);
-	gtk_label_set_selectable (GTK_LABEL (secondary_label), TRUE);
-	gtk_label_set_markup (GTK_LABEL (secondary_label), secondary_text);
-	
-	return(gui_dialog_make_from_widget(stock_id, primary_text, secondary_label));
-}
-
-GtkWidget *
-gui_dialog_make_from_widget(const gchar *stock_id, gchar *primary_text, GtkWidget *widget)
-{
-	GtkWidget *dialog, *image, *hhbox, *vvbox;
-	GtkWidget *primary_label;
-	gchar *str;
-
-	image = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_DIALOG);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
-	dialog = gtk_dialog_new();
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-	gtk_window_set_title (GTK_WINDOW (dialog), "");
-	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
-
-	primary_label = gtk_label_new (NULL);
-	gtk_label_set_line_wrap (GTK_LABEL (primary_label), TRUE);
-	gtk_label_set_use_markup (GTK_LABEL (primary_label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (primary_label), 0.0, 0.5);
-	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
-	str = g_strconcat("<span weight=\"bold\" size=\"larger\">", primary_text, "</span>", NULL);
-	gtk_label_set_markup (GTK_LABEL (primary_label), str);
-	g_free(str);
-
-	hhbox = gtk_hbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (hhbox), 5);
-	gtk_box_pack_start (GTK_BOX (hhbox), image, FALSE, FALSE, 0);
-	vvbox = gtk_vbox_new (FALSE, 12);
-	gtk_box_pack_start (GTK_BOX (hhbox), vvbox, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vvbox), primary_label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vvbox), widget, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hhbox, FALSE, FALSE, 0);
-
-	return(dialog);
-}
-
 void
 preview_wb_picked(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, RS_BLOB *rs)
 {
@@ -1027,6 +946,7 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 	/* Build iconbox */
 	rs->iconbox = rs_store_new();
 	g_signal_connect((gpointer) rs->iconbox, "thumb-activated", G_CALLBACK(icon_activated), rs);
+	g_signal_connect((gpointer) rs->iconbox, "group-activated", G_CALLBACK(group_activated), rs);
 
 	/* Catch window state changes (un/fullscreen) */
 	g_signal_connect((gpointer) rs->window, "window-state-event", G_CALLBACK(gui_fullscreen_iconbox_callback), rs->iconbox);
@@ -1039,7 +959,7 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 
 	/* Preview area */
 	rs->preview = rs_preview_widget_new();
-	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, PROFILE_DISPLAY));
+	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
 	rs_conf_get_color(CONF_PREBGCOLOR, &bgcolor);
 	rs_preview_widget_set_bgcolor(RS_PREVIEW_WIDGET(rs->preview), &bgcolor);
 	g_signal_connect(G_OBJECT(rs->preview), "wb-picked", G_CALLBACK(preview_wb_picked), rs);
