@@ -32,9 +32,6 @@ raw_mrw_walker(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 	guint tag=0, len=0;
 	gushort ushort_temp1=0;
 
-	guint next, toffset;
-	gushort ifd_num;
-
 	meta->make = MAKE_MINOLTA;
 
 	raw_get_uint(rawfile, offset+4, &rawstart);
@@ -49,15 +46,7 @@ raw_mrw_walker(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 		switch (tag)
 		{
 			case 0x00545457: /* TTW */
-				raw_init_file_tiff(rawfile, offset);
-				toffset = get_first_ifd_offset(rawfile);
-				do {
-					if (!raw_get_ushort(rawfile, toffset, &ifd_num)) break;
-					if (!raw_get_uint(rawfile, toffset+2+ifd_num*12, &next)) break;
-					raw_ifd_walker(rawfile, toffset, meta);
-					if (toffset == next) break; /* avoid infinite loops */
-					toffset = next;
-				} while (next>0);
+				rs_tiff_load_meta_from_rawfile(rawfile, offset, meta);
 				raw_reset_base(rawfile);
 				break;
 			case 0x00574247: /* WBG */
@@ -98,22 +87,10 @@ rs_mrw_load_thumb(const gchar *src)
 	RAWFILE *rawfile;
 	GdkPixbuf *pixbuf=NULL, *pixbuf2=NULL;
 	RS_METADATA meta;
-	gchar *thumbname;
 	guint start=0, length=0;
 
 	raw_init();
 
-	thumbname = rs_thumb_get_name(src);
-	if (thumbname)
-	{
-		if (g_file_test(thumbname, G_FILE_TEST_EXISTS))
-		{
-			pixbuf = gdk_pixbuf_new_from_file(thumbname, NULL);
-			g_free(thumbname);
-			if (pixbuf) return(pixbuf);
-		}
-	}
-
 	meta.thumbnail_start = 0;
 	meta.thumbnail_length = 0;
 	meta.preview_start = 0;
@@ -176,8 +153,6 @@ rs_mrw_load_thumb(const gchar *src)
 				pixbuf = pixbuf2;
 				break;
 		}
-		if (thumbname)
-			gdk_pixbuf_save(pixbuf, thumbname, "png", NULL, NULL);
 	}
 
 	raw_close_file(rawfile);
