@@ -33,7 +33,6 @@
 #include "ciff-meta.h"
 #include "mrw-meta.h"
 #include "x3f-meta.h"
-#include "panasonic.h"
 #include "rs-image.h"
 #include "gettext.h"
 #include "conf_interface.h"
@@ -49,6 +48,8 @@
 #include "rs-histogram.h"
 #include "rs-curve.h"
 #include "rs-photo.h"
+#include "rs-math.h"
+#include "rs-exif.h"
 
 static void photo_settings_changed(RS_PHOTO *photo, gint mask, RS_BLOB *rs);
 static void photo_spatial_changed(RS_PHOTO *photo, RS_BLOB *rs);
@@ -59,7 +60,7 @@ RS_FILETYPE *filetypes;
 
 static void
 rs_add_filetype(gchar *id, gint filetype, const gchar *ext, gchar *description,
-	RS_PHOTO *(*load)(const gchar *, gboolean),
+	RS_IMAGE16 *(*load)(const gchar *, gboolean),
 	GdkPixbuf *(*thumb)(const gchar *),
 	void (*load_meta)(const gchar *, RS_METADATA *),
 	gboolean (*save)(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms))
@@ -90,43 +91,47 @@ rs_init_filetypes(void)
 {
 	filetypes = NULL;
 	rs_add_filetype("cr2", FILETYPE_RAW, ".cr2", _("Canon CR2"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("crw", FILETYPE_RAW, ".crw", _("Canon CIFF"),
-		rs_photo_open_dcraw, rs_ciff_load_thumb, rs_ciff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_ciff_load_thumb, rs_ciff_load_meta, NULL);
 	rs_add_filetype("nef", FILETYPE_RAW, ".nef", _("Nikon NEF"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("mrw", FILETYPE_RAW, ".mrw", _("Minolta raw"),
-		rs_photo_open_dcraw, rs_mrw_load_thumb, rs_mrw_load_meta, NULL);
+		rs_image16_open_dcraw, rs_mrw_load_thumb, rs_mrw_load_meta, NULL);
 	rs_add_filetype("cr-tiff", FILETYPE_RAW, ".tif", _("Canon TIFF"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("arw", FILETYPE_RAW, ".arw", _("Sony"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+	rs_add_filetype("sr2", FILETYPE_RAW, ".sr2", _("Sony"),
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+	rs_add_filetype("sr2", FILETYPE_RAW, ".srf", _("Sony"),
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("kdc", FILETYPE_RAW, ".kdc", _("Kodak"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("kdc", FILETYPE_RAW, ".dcr", _("Kodak"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("x3f", FILETYPE_RAW, ".x3f", _("Sigma"),
-		rs_photo_open_dcraw, rs_x3f_load_thumb, NULL, NULL);
+		rs_image16_open_dcraw, rs_x3f_load_thumb, rs_x3f_load_meta, NULL);
 	rs_add_filetype("orf", FILETYPE_RAW, ".orf", "",
-		rs_photo_open_dcraw, rs_tiff_load_thumb, NULL, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("raw", FILETYPE_RAW, ".raw", _("Panasonic raw"),
-		rs_photo_open_dcraw, rs_panasonic_load_thumb, rs_panasonic_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("pef", FILETYPE_RAW, ".pef", _("Pentax raw"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("dng", FILETYPE_RAW, "dng", _("Adobe Digital negative"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("mef", FILETYPE_RAW, "mef", _("Mamiya"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("3fr", FILETYPE_RAW, "3fr", _("Hasselblad"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
+		rs_image16_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("jpeg", FILETYPE_JPEG, ".jpg", _("JPEG (Joint Photographic Experts Group)"),
-		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
+		rs_image16_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
 	rs_add_filetype("png", FILETYPE_PNG, ".png", _("PNG (Portable Network Graphics)"),
-		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
+		rs_image16_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
 	rs_add_filetype("tiff8", FILETYPE_TIFF8, ".tif", _("8-bit TIFF (Tagged Image File Format)"),
-		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
+		rs_image16_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
 	rs_add_filetype("tiff16", FILETYPE_TIFF16, ".tif", _("16-bit TIFF (Tagged Image File Format)"),
-		rs_photo_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
+		rs_image16_open_gdk, rs_thumb_gdk, NULL, rs_photo_save);
 	return;
 }
 
@@ -428,10 +433,12 @@ rs_metadata_new(void)
 	metadata->make = MAKE_UNKNOWN;
 	metadata->make_ascii = NULL;
 	metadata->model_ascii = NULL;
+	metadata->time_ascii = NULL;
+	metadata->timestamp = -1;
 	metadata->orientation = 0;
 	metadata->aperture = -1.0;
 	metadata->iso = 0;
-	metadata->shutterspeed = 1.0;
+	metadata->shutterspeed = -1.0;
 	metadata->thumbnail_start = 0;
 	metadata->thumbnail_length = 0;
 	metadata->preview_start = 0;
@@ -448,7 +455,6 @@ rs_metadata_new(void)
 	for(i=0;i<4;i++)
 		metadata->cam_mul[i] = 1.0f;
 	matrix4_identity(&metadata->adobe_coeff);
-	metadata->data = NULL;
 	return(metadata);
 }
 
@@ -459,10 +465,58 @@ rs_metadata_free(RS_METADATA *metadata)
 		g_free(metadata->make_ascii);
 	if (metadata->model_ascii)
 		g_free(metadata->model_ascii);
+	if (metadata->time_ascii)
+		g_free(metadata->time_ascii);
 	g_free(metadata);
 	return;
 }
 
+/**
+ * Load metadata from file
+ */
+gboolean
+rs_metadata_load(const gchar *filename, RS_METADATA *metadata)
+{
+	gboolean ret = FALSE;
+	RS_FILETYPE *filetype;
+
+	g_assert(filename != NULL);
+
+	filetype = rs_filetype_get(filename, TRUE);
+
+	if (filetype && filetype->load_meta)
+	{
+		filetype->load_meta(filename, metadata);
+		ret = TRUE;
+	}
+
+	return ret;
+}
+
+gchar *
+rs_metadata_get_short_description(RS_METADATA *metadata)
+{
+	GString *label = g_string_new("");
+	gchar *ret = NULL;
+
+	if (metadata->focallength>0)
+		g_string_append_printf(label, _("%dmm "), metadata->focallength);
+	if (metadata->shutterspeed > 0.0 && metadata->shutterspeed < 4) 
+		g_string_append_printf(label, _("%.1fs "), 1/metadata->shutterspeed);
+	else if (metadata->shutterspeed >= 4)
+		g_string_append_printf(label, _("1/%.0fs "), metadata->shutterspeed);
+	if (metadata->aperture!=0.0)
+		g_string_append_printf(label, _("F/%.1f "), metadata->aperture);
+	if (metadata->iso!=0)
+		g_string_append_printf(label, _("ISO%d"), metadata->iso);
+
+	ret = label->str;
+
+	g_string_free(label, FALSE);
+
+	return ret;
+}
+
 void
 rs_metadata_normalize_wb(RS_METADATA *meta)
 {
@@ -487,8 +541,9 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 	RS_IMAGE16 *image16;
 	gint quality = 100;
 	gboolean uncompressed_tiff = FALSE;
-	RS_COLOR_TRANSFORM *rct;
+	RSColorTransform *rct;
 	void *transform = NULL;
+	RS_EXIF_DATA *exif;
 
 	g_assert(RS_IS_PHOTO(photo));
 	g_assert(filename != NULL);
@@ -509,17 +564,17 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 
 	/* Initialize color transform */
 	rct = rs_color_transform_new();
-	rs_color_transform_set_from_settings(rct, photo->settings[snapshot], MASK_ALL);
-	rs_color_transform_set_output_format(rct, 8);
 	rs_color_transform_set_cms_transform(rct, transform);
 	rs_color_transform_set_adobe_matrix(rct, &photo->metadata->adobe_coeff);
+	rs_color_transform_set_from_settings(rct, photo->settings[snapshot], MASK_ALL);
+	rs_color_transform_set_output_format(rct, 8);
 
 	/* actually save */
 	switch (filetype)
 	{
 		case FILETYPE_JPEG:
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
-			rct->transform(rct, rsi->w, rsi->h, rsi->pixels,
+			rs_color_transform_transform(rct, rsi->w, rsi->h, rsi->pixels,
 				rsi->rowstride, gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf));
 
 			rs_conf_get_integer(CONF_EXPORT_JPEG_QUALITY, &quality);
@@ -528,12 +583,12 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 			else if (quality < 0)
 				quality = 0;
 
-			rs_jpeg_save(pixbuf, filename, quality, rs_cms_get_profile_filename(cms, PROFILE_EXPORT));
+			rs_jpeg_save(pixbuf, filename, quality, rs_cms_get_profile_filename(cms, CMS_PROFILE_EXPORT));
 			g_object_unref(pixbuf);
 			break;
 		case FILETYPE_PNG:
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
-			rct->transform(rct, rsi->w, rsi->h, rsi->pixels, rsi->rowstride,
+			rs_color_transform_transform(rct, rsi->w, rsi->h, rsi->pixels, rsi->rowstride,
 				gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf));
 			gdk_pixbuf_save(pixbuf, filename, "png", NULL, NULL);
 			g_object_unref(pixbuf);
@@ -541,31 +596,43 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 		case FILETYPE_TIFF8:
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
-			rct->transform(rct, rsi->w, rsi->h, rsi->pixels,
+			rs_color_transform_transform(rct, rsi->w, rsi->h, rsi->pixels,
 				rsi->rowstride, gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf));
-			rs_tiff8_save(pixbuf, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
+			rs_tiff8_save(pixbuf, filename, rs_cms_get_profile_filename(cms, CMS_PROFILE_EXPORT), uncompressed_tiff);
 			g_object_unref(pixbuf);
 			break;
 		case FILETYPE_TIFF16:
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
 			image16 = rs_image16_new(rsi->w, rsi->h, 3, 3);
 			rs_color_transform_set_output_format(rct, 16);
-			rct->transform(rct, rsi->w, rsi->h,
+			rs_color_transform_transform(rct, rsi->w, rsi->h,
 				rsi->pixels, rsi->rowstride,
 				image16->pixels, image16->rowstride*2);
-			rs_tiff16_save(image16, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
+			rs_tiff16_save(image16, filename, rs_cms_get_profile_filename(cms, CMS_PROFILE_EXPORT), uncompressed_tiff);
 			rs_image16_free(image16);
 			break;
 	}
 
 	rs_image16_free(rsi);
-	rs_color_transform_free(rct);
+	g_object_unref(rct);
 
 	photo->exported = TRUE;
-	rs_cache_save(photo);
+	rs_cache_save(photo, MASK_ALL);
 
 	/* Set the exported flag */
 	rs_store_set_flags(NULL, photo->filename, NULL, NULL, &photo->exported);
+
+	/* Save exif for JPEG files */
+	if (filetype == FILETYPE_JPEG)
+	{
+		exif = rs_exif_load_from_file(photo->filename);
+		if (exif)
+		{
+			rs_exif_add_to_file(exif, filename);
+			rs_exif_free(exif);
+		}
+	}
+
 	return(TRUE);
 }
 
@@ -701,27 +768,36 @@ static GdkPixbuf *
 rs_thumb_gdk(const gchar *src)
 {
 	GdkPixbuf *pixbuf=NULL;
-	gchar *thumbname;
 
-	thumbname = rs_thumb_get_name(src);
+	pixbuf = gdk_pixbuf_new_from_file_at_size(src, 128, 128, NULL);
+
+	return(pixbuf);
+}
+
+GdkPixbuf *
+rs_load_thumb(RS_FILETYPE *filetype, const gchar *src)
+{
+	GdkPixbuf * pixbuf = NULL;
+	gchar *thumbname = rs_thumb_get_name(src);
 
 	if (thumbname)
 	{
-		if (g_file_test(thumbname, G_FILE_TEST_EXISTS))
+		pixbuf = gdk_pixbuf_new_from_file(thumbname, NULL);
+
+		if (!pixbuf && filetype->thumb)
 		{
-			pixbuf = gdk_pixbuf_new_from_file(thumbname, NULL);
-			g_free(thumbname);
-		}
-		else
-		{
-			pixbuf = gdk_pixbuf_new_from_file_at_size(src, 128, 128, NULL);
-			gdk_pixbuf_save(pixbuf, thumbname, "png", NULL, NULL);
-			g_free(thumbname);
+			pixbuf = filetype->thumb(src);
+
+			if (pixbuf)
+				gdk_pixbuf_save(pixbuf, thumbname, "png", NULL, NULL);
 		}
+
+		g_free(thumbname);
 	}
-	else
-		pixbuf = gdk_pixbuf_new_from_file_at_size(src, 128, 128, NULL);
-	return(pixbuf);
+	else if (filetype->thumb)
+		pixbuf = filetype->thumb(src);
+
+	return pixbuf;
 }
 
 void
@@ -736,12 +812,12 @@ rs_white_black_point(RS_BLOB *rs)
 		gdouble blackpoint;
 		gdouble whitepoint;
 		guint total = 0;
-		RS_COLOR_TRANSFORM *rct;
+		RSColorTransform *rct;
 
 		rct = rs_color_transform_new();
 		rs_color_transform_set_from_settings(rct, rs->photo->settings[rs->current_setting], MASK_ALL ^ MASK_CURVE);
 		rs_color_transform_make_histogram(rct, rs->histogram_dataset, hist);
-		rs_color_transform_free(rct);
+		g_object_unref(rct);
 
 		// calculate black point
 		while(i < 256) {
@@ -1021,19 +1097,145 @@ rs_has_gimp(gint major, gint minor, gint micro) {
 	return retval;
 }
 
+/**
+ * This is a very simple regression test for Rawstudio. Filenames will be read
+ * from "testimages" in the current directory, one filename per line, and a
+ * small series of tests will be carried out for each filename. Output can be
+ * piped to a file for further processing.
+ */
+void
+test()
+{
+	gchar *filename, *basename;
+	GdkPixbuf *pixbuf;
+	GIOChannel *io = g_io_channel_new_file("testimages", "r", NULL);
+	gint sum, good = 0, bad = 0;
+
+	printf("basename, load, filetype, thumb, meta, make, a-make, a-model, aperture, iso, s-speed, wb, f-length\n");
+	while (G_IO_STATUS_EOF != g_io_channel_read_line(io, &filename, NULL, NULL, NULL))
+	{
+		gboolean filetype_ok = FALSE;
+		gboolean load_ok = FALSE;
+		gboolean thumbnail_ok = FALSE;
+		gboolean load_meta_ok = FALSE;
+		gboolean make_ok = FALSE;
+		gboolean make_ascii_ok = FALSE;
+		gboolean model_ascii_ok = FALSE;
+		gboolean aperture_ok = FALSE;
+		gboolean iso_ok = FALSE;
+		gboolean shutterspeed_ok = FALSE;
+		gboolean wb_ok = FALSE;
+		gboolean focallength_ok = FALSE;
+
+		RS_METADATA *metadata = rs_metadata_new();
+
+		g_strstrip(filename);
+		RS_FILETYPE *filetype = rs_filetype_get(filename, TRUE);
+
+		if (filetype)
+		{
+			filetype_ok = TRUE;
+			if (filetype->load)
+			{
+				RS_PHOTO *photo = NULL;
+				photo = rs_photo_load_from_file(filename, TRUE);
+				if (photo)
+				{
+					load_ok = TRUE;
+					g_object_unref(photo);
+				}
+			}
+
+			pixbuf = rs_load_thumb(filetype, filename);
+			if (pixbuf)
+			{
+				thumbnail_ok = TRUE;
+				g_object_unref(pixbuf);
+			}
+
+			if (filetype->load_meta)
+			{
+				load_meta_ok = TRUE;
+				filetype->load_meta(filename, metadata);
+				if (metadata->make != MAKE_UNKNOWN)
+					make_ok = TRUE;
+				if (metadata->make_ascii != NULL)
+					make_ascii_ok = TRUE;
+				if (metadata->model_ascii != NULL)
+					model_ascii_ok = TRUE;
+				if (metadata->aperture > 0.0)
+					aperture_ok = TRUE;
+				if (metadata->iso > 0)
+					iso_ok = TRUE;
+				if (metadata->shutterspeed > 1.0)
+					shutterspeed_ok = TRUE;
+				if (metadata->cam_mul[0] > 0.1 && metadata->cam_mul[0] != 1.0)
+					wb_ok = TRUE;
+				if (metadata->focallength > 0.0)
+					focallength_ok = TRUE;
+			}
+
+		}
+
+		basename = g_path_get_basename(filename);
+		printf("%s, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d\n",
+			basename,
+			load_ok,
+			filetype_ok,
+			thumbnail_ok,
+			load_meta_ok,
+			make_ok,
+			make_ascii_ok,
+			model_ascii_ok,
+			aperture_ok,
+			iso_ok,
+			shutterspeed_ok,
+			wb_ok,
+			focallength_ok
+		);
+		sum = load_ok
+			+filetype_ok
+			+thumbnail_ok
+			+load_meta_ok
+			+make_ok
+			+make_ascii_ok
+			+model_ascii_ok
+			+aperture_ok
+			+iso_ok
+			+shutterspeed_ok
+			+wb_ok
+			+focallength_ok;
+
+		good += sum;
+		bad += (12-sum);
+
+		g_free(basename);
+
+		g_free(filename);
+		rs_metadata_free(metadata);
+	}
+	printf("Passed: %d Failed: %d (%d%%)\n", good, bad, (good*100)/(good+bad));
+	g_io_channel_shutdown(io, TRUE, NULL);
+	exit(0);
+}
+
 int
 main(int argc, char **argv)
 {
 	RS_BLOB *rs;
 	int optimized = 1;
+	gboolean do_test = FALSE;
 	int opt;
 	gboolean use_system_theme = DEFAULT_CONF_USE_SYSTEM_THEME;
 
-	while ((opt = getopt(argc, argv, "n")) != -1) {
+	while ((opt = getopt(argc, argv, "nt")) != -1) {
 		switch (opt) {
 		case 'n':
 			optimized = 0;
 			break;
+		case 't':
+			do_test = TRUE;
+			break;
 		}
 	}
 
@@ -1065,7 +1267,11 @@ main(int argc, char **argv)
 
 	rs = rs_new();
 	rs->queue->cms = rs->cms = rs_cms_init();
-	gui_init(argc, argv, rs);
+
+	if (do_test)
+		test();
+	else
+		gui_init(argc, argv, rs);
 
 	/* This is so fucking evil, but Rawstudio will deadlock in some GTK atexit() function from time to time :-/ */
 	_exit(0);
