@@ -32,6 +32,7 @@
 #include "gtk-interface.h"
 #include "gtk-save-dialog.h"
 #include "gtk-progress.h"
+#include "gtk-helper.h"
 #include "rs-external-editor.h"
 #include "rs-cache.h"
 #include "rs-preview-widget.h"
@@ -99,6 +100,7 @@ ACTION(photo_menu)
 #ifndef EXPERIMENTAL
 	rs_core_action_group_set_visibility("Group", FALSE);
 	rs_core_action_group_set_visibility("Ungroup", FALSE);
+	rs_core_action_group_set_visibility("AutoGroup", FALSE);
 #endif
 	g_list_free(selected);
 }
@@ -115,7 +117,7 @@ ACTION(batch_menu)
 	photos_selected = (RS_IS_PHOTO(rs->photo) || (num_selected > 0));
 
 	rs_core_action_group_set_sensivity("AddToBatch", photos_selected);
-	rs_core_action_group_set_sensivity("RemoveFromBatch", photos_selected);
+	rs_core_action_group_set_sensivity("RemoveFromBatch", photos_selected && rs->photo && rs_batch_exists_in_queue(rs->queue, rs->photo->filename, rs->current_setting));
 	rs_core_action_group_set_sensivity("ProcessBatch", (rs_batch_num_entries(rs->queue)>0));
 	g_list_free(selected);
 }
@@ -338,7 +340,7 @@ ACTION(copy_settings)
 
 ACTION(paste_settings)
 {
-	gint mask;
+	gint mask = 0xffffff;
 	
 	GtkWidget *dialog, *cb_box;
 	GtkWidget *cb_exposure, *cb_saturation, *cb_hue, *cb_contrast, *cb_whitebalance, *cb_curve, *cb_sharpen;
@@ -417,6 +419,7 @@ ACTION(paste_settings)
 			gint cur;
 			GList *selected = NULL;
 			gint num_selected;
+			guint new_mask;
 
 			/* Apply to all selected photos */
 			selected = rs_store_get_selected_names(rs->store);
@@ -428,9 +431,8 @@ ACTION(paste_settings)
 				photo->filename = g_strdup(g_list_nth_data(selected, cur));
 				if ((filetype = rs_filetype_get(photo->filename, TRUE)))
 				{
-					if (filetype->load_meta)
+					if (rs_metadata_load(photo->filename, photo->metadata))
 					{
-						filetype->load_meta(photo->filename, photo->metadata);
 						switch (photo->metadata->orientation)
 						{
 							case 90: ORIENTATION_90(photo->orientation);
@@ -441,9 +443,9 @@ ACTION(paste_settings)
 								break;
 						}
 					}
-					rs_cache_load(photo);
+					new_mask = rs_cache_load(photo);
 					rs_settings_double_copy(rs->settings_buffer, photo->settings[rs->current_setting], mask);
-					rs_cache_save(photo);
+					rs_cache_save(photo, new_mask | mask);
 				}
 				g_object_unref(photo);
 			}
@@ -551,6 +553,11 @@ ACTION(ungroup_photos)
 	rs_store_ungroup_photos(rs->store);
 }
 
+ACTION(auto_group_photos)
+{
+	rs_store_auto_group(rs->store);
+}
+
 ACTION(previous_photo)
 {
 	gchar *current_filename = NULL;
@@ -632,7 +639,7 @@ ACTION(add_to_batch)
 
 	if (RS_IS_PHOTO(rs->photo) && num_selected == 1)
 	{
-		rs_cache_save(rs->photo);
+		rs_cache_save(rs->photo, MASK_ALL);
 
 		if (rs_batch_add_to_queue(rs->queue, rs->photo->filename, rs->current_setting))
 			g_string_printf(gs, _(" %s added to batch queue"), rs->photo->filename);
@@ -714,7 +721,7 @@ ACTION(add_view_to_batch)
 
 		/* Save settings of current photo just to be sure */
 		if (rs->photo)
-			rs_cache_save(rs->photo);
+			rs_cache_save(rs->photo, MASK_ALL);
 
 		g_string_printf(gs, _("%d photos added to batch queue"),
 			((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cb_a))) ? num_selected : 0)
@@ -760,7 +767,7 @@ ACTION(about)
 	gtk_show_about_dialog(GTK_WINDOW(rawstudio_window),
 		"authors", authors,
 		"artists", artists,
-		"translator-credits", "Simone Contini\nPaweł Gołaszewski\nAlexandre Prokoudine\nJakub Friedl\nCarsten Mathaes\nEdouard Gomez\nMartin Egger\nKrzysztof Kościuszkiewicz\nEinar Ryeng\nOlli Hänninen\nCarlos Dávila",
+		"translator-credits", "Simone Contini\nPaweł Gołaszewski\nAlexandre Prokoudine\nJakub Friedl\nCarsten Mathaes\nEdouard Gomez\nMartin Egger\nKrzysztof Kościuszkiewicz\nEinar Ryeng\nOlli Hänninen\nCarlos Dávila\nPatrik Jarl\nOlav Lavell\nRafael Sachetto Oliveira\nPaco Rivière",
 		"comments", _("A raw image converter for GTK+/GNOME"),
 		"version", VERSION,
 		"website", "http://rawstudio.org/",
@@ -827,6 +834,7 @@ rs_get_core_action_group(RS_BLOB *rs)
 	{ "Unstraighten", NULL, _("_Unstraighten"), NULL, NULL, ACTION_CB(unstraighten) },
 	{ "Group", NULL, _("_Group"), NULL, NULL, ACTION_CB(group_photos) },
 	{ "Ungroup", NULL, _("_Ungroup"), NULL, NULL, ACTION_CB(ungroup_photos) },
+	{ "AutoGroup", NULL, _("_Auto group"), NULL, NULL, ACTION_CB(auto_group_photos) },
 
 	/* View menu */
 	{ "PreviousPhoto", GTK_STOCK_GO_BACK, _("_Previous photo"), "<control>Left", NULL, ACTION_CB(previous_photo) },
@@ -838,7 +846,7 @@ rs_get_core_action_group(RS_BLOB *rs)
 	{ "AddToBatch", GTK_STOCK_ADD, _("_Add to batch queue"), "<control>B", NULL, ACTION_CB(add_to_batch) },
 	{ "AddViewToBatch", NULL, _("_Add current view to queue"), NULL, NULL, ACTION_CB(add_view_to_batch) },
 	{ "RemoveFromBatch", GTK_STOCK_REMOVE, _("_Remove from batch queue"), "<control><alt>B", NULL, ACTION_CB(remove_from_batch) },
-	{ "ProcessBatch", GTK_STOCK_CONVERT, _("_Start"), NULL, NULL, ACTION_CB(ProcessBatch) },
+	{ "ProcessBatch", GTK_STOCK_EXECUTE, _("_Start"), NULL, NULL, ACTION_CB(ProcessBatch) },
 
 	/* help menu */
 	{ "About", GTK_STOCK_ABOUT, _("_About"), NULL, NULL, ACTION_CB(about) },
