@@ -32,13 +32,14 @@
 #include "rs-cache.h"
 #include "rs-color-transform.h"
 #include "rs-image.h"
+#include "rs-photo.h"
 
 extern GtkWindow *rawstudio_window;
 
-static gboolean batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id);
 static GtkWidget *make_batchview(RS_QUEUE *queue);
 static void size_update_infolabel(RS_QUEUE *queue);
 static gchar *batch_queue_filename = NULL;
+static void batch_queue_update_sensivity(RS_QUEUE *queue);
 
 static void
 batch_queue_save(RS_QUEUE *queue)
@@ -200,7 +201,7 @@ gboolean
 rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting_id)
 {
 	gboolean ret = FALSE;
-	if (!batch_exists_in_queue(queue, filename, setting_id))
+	if (!rs_batch_exists_in_queue(queue, filename, setting_id))
 	{
 		gchar *filename_short, *setting_id_abc;
 		RS_FILETYPE *filetype;
@@ -229,8 +230,7 @@ rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting
 			missing_thumb = gtk_widget_render_icon(GTK_WIDGET(rawstudio_window),
 				GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
 
-			if (filetype->thumb)
-				pixbuf = filetype->thumb(filename);
+			pixbuf = rs_load_thumb(filetype, filename);
 			if (pixbuf)
 			{
 				gint w,h,temp,size = 48;
@@ -260,7 +260,7 @@ rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting
 			}
 			g_object_unref(missing_thumb);
 
-			if (!batch_exists_in_queue(queue, filename, setting_id))
+			if (!rs_batch_exists_in_queue(queue, filename, setting_id))
 			{
 				GtkTreeIter iter;
 
@@ -281,6 +281,8 @@ rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting
 
 	batch_queue_save(queue);
 
+	batch_queue_update_sensivity(queue);
+
 	return ret;
 }
 
@@ -313,16 +315,22 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 				}
 			}
 			g_free(filename_temp);
+
+			/* Break out of the loop if we got a hit */
+			if (ret)
+				break;
 		} while (gtk_tree_model_iter_next(queue->list, &iter));
 	}
 
 	batch_queue_save(queue);
 
+	batch_queue_update_sensivity(queue);
+
 	return ret;
 }
 
-static gboolean
-batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id)
+gboolean
+rs_batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id)
 {
 	gboolean ret = FALSE;
 	GtkTreeIter iter;
@@ -372,7 +380,6 @@ void
 rs_batch_process(RS_QUEUE *queue)
 {
 	RS_PHOTO *photo = NULL;
-	RS_FILETYPE *filetype;
 	RS_IMAGE16 *image;
 	GtkTreeIter iter;
 	gchar *filename_in;
@@ -390,7 +397,7 @@ rs_batch_process(RS_QUEUE *queue)
 	GtkWidget *cancel;
 	gboolean abort_render = FALSE;
 	gboolean fullscreen = FALSE;
-	RS_COLOR_TRANSFORM *rct = rs_color_transform_new();
+	RSColorTransform *rct = rs_color_transform_new();
 	GTimeVal start_time;
 	GTimeVal now_time = {0,0};
 	gint time, eta;
@@ -476,7 +483,7 @@ rs_batch_process(RS_QUEUE *queue)
 			RS_QUEUE_ELEMENT_FILENAME, &filename_in,
 			RS_QUEUE_ELEMENT_SETTING_ID, &setting_id,
 			-1);
-		if ((filetype = rs_filetype_get(filename_in, TRUE)))
+		if (rs_filetype_get(filename_in, TRUE))
 		{
 			basename = g_path_get_basename(filename_in);
 			g_string_printf(status, _("Loading %s ..."), basename);
@@ -484,12 +491,11 @@ rs_batch_process(RS_QUEUE *queue)
 			while (gtk_events_pending()) gtk_main_iteration();
 			g_free(basename);
 
-			photo = filetype->load(filename_in, FALSE);
+			photo = rs_photo_load_from_file(filename_in, FALSE);
 			rs_image16_demosaic(photo->input, RS_DEMOSAIC_PPG);
 			if (photo)
 			{
-				if (filetype->load_meta)
-					filetype->load_meta(filename_in, photo->metadata);
+				rs_metadata_load(filename_in, photo->metadata);
 				filename = g_string_new(queue->directory);
 				g_string_append(filename, G_DIR_SEPARATOR_S);
 				g_string_append(filename, queue->filename);
@@ -523,7 +529,7 @@ rs_batch_process(RS_QUEUE *queue)
 
 				/* Render preview image */
 				rs_color_transform_set_from_settings(rct, photo->settings[setting_id], MASK_ALL);
-				rct->transform(rct, image->w, image->h, image->pixels,
+				rs_color_transform_transform(rct, image->w, image->h, image->pixels,
 					image->rowstride, gdk_pixbuf_get_pixels(pixbuf),
 					gdk_pixbuf_get_rowstride(pixbuf));
 				gtk_image_set_from_pixbuf((GtkImage *) preview, pixbuf);
@@ -548,20 +554,21 @@ rs_batch_process(RS_QUEUE *queue)
 
 		g_get_current_time(&now_time);
 	}
-	rs_color_transform_free(rct);
+	g_object_unref(rct);
 	gtk_widget_destroy(window);
 
 	/* Restore fullscreen state if needed */
 	if (fullscreen)	
 		gtk_window_fullscreen(rawstudio_window);
 	gtk_widget_show_all(GTK_WIDGET(rawstudio_window));
+
+	batch_queue_update_sensivity(queue);
 }
 
-void
-rs_batch_start_queue(RS_QUEUE *queue)
+static void
+cursor_changed(GtkTreeView *tree_view, gpointer user_data)
 {
-	rs_batch_process(queue);
-	return;
+	batch_queue_update_sensivity((RS_QUEUE *) user_data);
 }
 
 static GtkWidget *
@@ -609,6 +616,7 @@ make_batchview(RS_QUEUE *queue)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view), column_filename);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (view), column_setting_id);
 
+	g_signal_connect(G_OBJECT(view), "cursor-changed", G_CALLBACK(cursor_changed), queue);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW (view), FALSE);
 
 	return scroller;
@@ -632,6 +640,7 @@ batch_button_remove_clicked(GtkWidget *button, RS_QUEUE *queue)
 			batch_queue_save(queue);
 		}
 	}
+	batch_queue_update_sensivity(queue);
 	return;
 }
 
@@ -640,40 +649,65 @@ batch_button_remove_all_clicked(GtkWidget *button, RS_QUEUE *queue)
 {
 	gtk_list_store_clear(GTK_LIST_STORE(queue->list));
 	batch_queue_save(queue);
+	batch_queue_update_sensivity(queue);
 	return;
 }
 
 static void
 batch_button_start_clicked(GtkWidget *button, RS_QUEUE *queue)
 {
-	rs_batch_start_queue(queue);
+	rs_batch_process(queue);
 	return;
 }
 
+static void
+batch_queue_update_sensivity(RS_QUEUE *queue)
+{
+	GtkTreePath *selected_path;
+	GtkTreeIter iter;
+
+	/* If we have any entries, enable "Start" and "Remove all" */
+	if (gtk_tree_model_get_iter_first(queue->list, &iter))
+	{
+		gtk_widget_set_sensitive(queue->start_button, TRUE);
+		gtk_widget_set_sensitive(queue->remove_all_button, TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive(queue->start_button, FALSE);
+		gtk_widget_set_sensitive(queue->remove_all_button, FALSE);
+	}
+
+	/* If anything is selected, enable "Remove" */
+	gtk_tree_view_get_cursor(queue->view, &selected_path, NULL);
+	if(selected_path)
+	{
+		gtk_widget_set_sensitive(queue->remove_button, TRUE);
+		gtk_tree_path_free(selected_path);
+	}
+	else
+		gtk_widget_set_sensitive(queue->remove_button, FALSE);
+}
+
 GtkWidget *
 make_batchbuttons(RS_QUEUE *queue)
 {
 		GtkWidget *box;
-		GtkWidget *start_button;
-		GtkWidget *remove_button;
-		GtkWidget *remove_all_button;
 
 		box = gtk_hbox_new(FALSE,4);
 
-		start_button = gtk_button_new_with_label(_("Start"));
-		g_signal_connect ((gpointer) start_button, "clicked", G_CALLBACK (batch_button_start_clicked), queue);
+		queue->start_button = gui_button_new_from_stock_with_label(GTK_STOCK_EXECUTE, _("Start"));
+		g_signal_connect ((gpointer) queue->start_button, "clicked", G_CALLBACK (batch_button_start_clicked), queue);
 
-		remove_button = gtk_button_new();
-		gtk_button_set_label(GTK_BUTTON(remove_button), _("Remove"));
-		g_signal_connect ((gpointer) remove_button, "clicked", G_CALLBACK (batch_button_remove_clicked), queue);
+		queue->remove_button = gui_button_new_from_stock_with_label(GTK_STOCK_REMOVE, _("Remove"));
+		g_signal_connect ((gpointer) queue->remove_button, "clicked", G_CALLBACK (batch_button_remove_clicked), queue);
 
-		remove_all_button = gtk_button_new();
-		gtk_button_set_label(GTK_BUTTON(remove_all_button), _("Remove all"));
-		g_signal_connect ((gpointer) remove_all_button, "clicked", G_CALLBACK (batch_button_remove_all_clicked), queue);
+		queue->remove_all_button = gui_button_new_from_stock_with_label(GTK_STOCK_REMOVE, _("Remove all"));
+		g_signal_connect ((gpointer) queue->remove_all_button, "clicked", G_CALLBACK (batch_button_remove_all_clicked), queue);
 
-		gtk_box_pack_start(GTK_BOX (box), start_button, FALSE, FALSE, 0);
-		gtk_box_pack_start(GTK_BOX (box), remove_button, FALSE, FALSE, 0);
-		gtk_box_pack_start(GTK_BOX (box), remove_all_button, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX (box), queue->start_button, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX (box), queue->remove_button, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX (box), queue->remove_all_button, FALSE, FALSE, 0);
 
 		return box;
 }
@@ -945,6 +979,8 @@ make_batchbox(RS_QUEUE *queue)
 	gtk_box_pack_start (GTK_BOX (batchbox), make_batchbuttons(queue), FALSE, FALSE, 0);
 	batch_queue_load(queue);
 
+	batch_queue_update_sensivity(queue);
+
 	return batchbox;
 }
 
