@@ -23,11 +23,12 @@
 #include "rawstudio.h"
 #include "rs-cache.h"
 #include "rs-photo.h"
+#include "rs-utils.h"
 
 /* This will be written to XML files for making backward compatibility easier to implement */
 #define CACHEVERSION 2
 
-static void rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur);
+static guint rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur);
 
 gchar *
 rs_cache_get_name(const gchar *src)
@@ -52,12 +53,14 @@ rs_cache_get_name(const gchar *src)
 }
 
 void
-rs_cache_save(RS_PHOTO *photo)
+rs_cache_save(RS_PHOTO *photo, guint mask)
 {
 	gint id, i;
 	xmlTextWriterPtr writer;
 	gchar *cachename;
 
+	if (!photo->filename) return;
+
 	cachename = rs_cache_get_name(photo->filename);
 	if (!cachename) return;
 	writer = xmlNewTextWriterFilename(cachename, 0); /* fixme, check for errors */
@@ -79,25 +82,32 @@ rs_cache_save(RS_PHOTO *photo)
 			photo->crop->x1, photo->crop->y1,
 			photo->crop->x2, photo->crop->y2);
 	}
-	for(id=0;id<3;id++)
+	for(id=0;id<3&&mask>0;id++)
 	{
 		xmlTextWriterStartElement(writer, BAD_CAST "settings");
 		xmlTextWriterWriteFormatAttribute(writer, BAD_CAST "id", "%d", id);
-		xmlTextWriterWriteFormatElement(writer, BAD_CAST "exposure", "%f",
-			photo->settings[id]->exposure);
+		if (mask & MASK_EXPOSURE)
+			xmlTextWriterWriteFormatElement(writer, BAD_CAST "exposure", "%f",
+				photo->settings[id]->exposure);
+		if (mask & MASK_SATURATION)
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "saturation", "%f",
 			photo->settings[id]->saturation);
+		if (mask & MASK_HUE)
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "hue", "%f",
 			photo->settings[id]->hue);
+		if (mask & MASK_CONTRAST)
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "contrast", "%f",
 			photo->settings[id]->contrast);
+		if (mask & MASK_WARMTH)
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "warmth", "%f",
 			photo->settings[id]->warmth);
+		if (mask & MASK_TINT)
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "tint", "%f",
 			photo->settings[id]->tint);
+		if (mask & MASK_SHARPEN)
 		xmlTextWriterWriteFormatElement(writer, BAD_CAST "sharpen", "%f",
 			photo->settings[id]->sharpen);
-		if (photo->settings[id]->curve_nknots > 0)
+		if (mask & MASK_CURVE && photo->settings[id]->curve_nknots > 0)
 		{
 			xmlTextWriterStartElement(writer, BAD_CAST "curve");
 			xmlTextWriterWriteFormatAttribute(writer, BAD_CAST "num", "%d", photo->settings[id]->curve_nknots);
@@ -115,9 +125,10 @@ rs_cache_save(RS_PHOTO *photo)
 	return;
 }
 
-static void
+static guint
 rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 {
+	guint mask = 0;
 	xmlChar *val;
 	gdouble *target=NULL;
 	xmlNodePtr curve = NULL;
@@ -125,19 +136,40 @@ rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 	{
 		target = NULL;
 		if ((!xmlStrcmp(cur->name, BAD_CAST "exposure")))
+		{
+			mask |= MASK_EXPOSURE;
 			target = &rss->exposure;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "saturation")))
+		{
+			mask |= MASK_SATURATION;
 			target = &rss->saturation;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "hue")))
+		{
+			mask |= MASK_HUE;
 			target = &rss->hue;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "contrast")))
+		{
+			mask |= MASK_CONTRAST;
 			target = &rss->contrast;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "warmth")))
+		{
+			mask |= MASK_WARMTH;
 			target = &rss->warmth;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "tint")))
+		{
+			mask |= MASK_TINT;
 			target = &rss->tint;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "sharpen")))
+		{
+			mask |= MASK_SHARPEN;
 			target = &rss->sharpen;
+		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "curve")))
 		{
 			gchar **vals;
@@ -157,12 +189,13 @@ rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 			{
 				if ((!xmlStrcmp(curve->name, BAD_CAST "knot")))
 				{
+					mask |= MASK_CURVE;
 					val = xmlNodeListGetString(doc, curve->xmlChildrenNode, 1);
 					vals = g_strsplit((gchar *)val, " ", 4);
 					if (vals[0] && vals[1])
 					{
-						x = g_strtod(vals[0], NULL);
-						y = g_strtod(vals[1], NULL);
+						x = rs_atof(vals[0]);
+						y = rs_atof(vals[1]);
 						rss->curve_knots[rss->curve_nknots*2+0] = x;
 						rss->curve_knots[rss->curve_nknots*2+1] = y;
 						rss->curve_nknots++;
@@ -178,16 +211,19 @@ rs_cache_load_setting(RS_SETTINGS_DOUBLE *rss, xmlDocPtr doc, xmlNodePtr cur)
 		if (target)
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			*target =  g_strtod((gchar *) val, NULL);
+			*target =  rs_atof((gchar *) val);
 			xmlFree(val);
 		}
 		cur = cur->next;
 	}
+
+	return mask;
 }
 
-gboolean
+guint
 rs_cache_load(RS_PHOTO *photo)
 {
+	guint mask = 0;
 	xmlDocPtr doc;
 	xmlNodePtr cur;
 	xmlChar *val;
@@ -197,11 +233,14 @@ rs_cache_load(RS_PHOTO *photo)
 	RS_SETTINGS_DOUBLE *settings;
 
 	cachename = rs_cache_get_name(photo->filename);
-	if (!cachename) return(FALSE);
+	if (!cachename) return mask;
 	if (!g_file_test(cachename, G_FILE_TEST_IS_REGULAR)) return FALSE;
 	photo->exported = FALSE;
 	doc = xmlParseFile(cachename);
-	if(doc==NULL) return(FALSE);
+	if(doc==NULL) return mask;
+
+	/* Return something if the file exists */
+	mask = 0x80000000;
 
 	cur = xmlDocGetRootElement(doc);
 
@@ -223,7 +262,7 @@ rs_cache_load(RS_PHOTO *photo)
 			if (id>2) id=0;
 			if (id<0) id=0;
 			settings = rs_settings_double_new();
-			rs_cache_load_setting(settings, doc, cur->xmlChildrenNode);
+			mask |= rs_cache_load_setting(settings, doc, cur->xmlChildrenNode);
 			rs_photo_apply_settings_double(photo, id, settings, MASK_ALL);
 			rs_settings_double_free(settings);
 		}
@@ -242,7 +281,7 @@ rs_cache_load(RS_PHOTO *photo)
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "angle")))
 		{
 			val = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
-			photo->angle = g_strtod((gchar *) val, NULL);
+			photo->angle = rs_atof((gchar *) val);
 			xmlFree(val);
 		}
 		else if ((!xmlStrcmp(cur->name, BAD_CAST "exported")))
@@ -294,7 +333,7 @@ rs_cache_load(RS_PHOTO *photo)
 
 	xmlFreeDoc(doc);
 	g_free(cachename);
-	return(TRUE);
+	return mask;
 }
 
 void
@@ -356,6 +395,7 @@ void
 rs_cache_save_flags(const gchar *filename, const guint *priority, const gboolean *exported)
 {
 	RS_PHOTO *photo;
+	guint mask;
 
 	g_assert(filename != NULL);
 
@@ -365,14 +405,14 @@ rs_cache_save_flags(const gchar *filename, const guint *priority, const gboolean
 	photo = rs_photo_new();
 	photo->filename = (gchar *) filename;
 
-	if (rs_cache_load(photo))
+	if ((mask = rs_cache_load(photo)))
 	{
 		/* If we got a cache file, save as normal */
 		if (priority)
 			photo->priority = *priority;
 		if (exported)
 			photo->exported = *exported;
-		rs_cache_save(photo);
+		rs_cache_save(photo, mask);
 	}
 	else
 	{
