@@ -21,8 +21,8 @@
 #define RS_CACHE_H
 
 extern gchar *rs_cache_get_name(const gchar *src);
-extern void rs_cache_save(RS_PHOTO *photo);
-extern gboolean rs_cache_load(RS_PHOTO *photo);
+extern void rs_cache_save(RS_PHOTO *photo, guint mask);
+extern guint rs_cache_load(RS_PHOTO *photo);
 extern void rs_cache_load_quick(const gchar *filename, gint *priority, gboolean *exported);
 extern void rs_cache_save_flags(const gchar *filename, const guint *priority, const gboolean *exported);
 
