@@ -27,6 +27,7 @@
 #include "rs-curve.h"
 #include "rs-color-transform.h"
 #include "gtk-interface.h"
+#include "rs-utils.h"
 
 struct _RSCurveWidget
 {
@@ -42,7 +43,7 @@ struct _RSCurveWidget
 	/* For drawing the histogram */
 	guint histogram_data[4][256];
 	guchar *bg_buffer;
-	RS_COLOR_TRANSFORM *rct;
+	RSColorTransform *rct;
 	RS_SETTINGS_DOUBLE *settings;
 };
 
@@ -398,10 +399,19 @@ rs_curve_widget_load(RSCurveWidget *curve, const gchar *filename)
 		{
 			gchar **vals;
 			gfloat x,y;
+			guint nknots;
+			gfloat *knots;
 			xmlNodePtr curknot = NULL;
 
+			rs_curve_widget_get_knots(curve, &knots, &nknots);
+
+			while (nknots--)
+			{
+				rs_spline_delete(curve->spline, nknots);	
+			}
+			g_free(knots);
+
 			curknot = cur->xmlChildrenNode;
-			rs_curve_widget_reset(curve);
 			while (curknot)
 			{
 				if ((!xmlStrcmp(curknot->name, BAD_CAST "AnchorXY")))
@@ -410,8 +420,8 @@ rs_curve_widget_load(RSCurveWidget *curve, const gchar *filename)
 					vals = g_strsplit((gchar *)val, " ", 4);
 					if (vals[0] && vals[1])
 					{
-						x = g_strtod(vals[0], NULL);
-						y = g_strtod(vals[1], NULL);
+						x = rs_atof(vals[0]);
+						y = rs_atof(vals[1]);
 						rs_curve_widget_add_knot(curve, x,y);
 					}
 					g_strfreev(vals);
