@@ -22,9 +22,9 @@
 #include "color.h"
 #include "rs-cache.h"
 #include "rs-curve.h"
+#include "rs-preload.h"
 
 static void rs_photo_class_init (RS_PHOTOClass *klass);
-void rs_photo_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_PHOTO *photo);
 
 G_DEFINE_TYPE (RS_PHOTO, rs_photo, G_TYPE_OBJECT);
 
@@ -197,6 +197,36 @@ rs_photo_get_crop(RS_PHOTO *photo)
 	return photo->crop;
 }
 
+/**
+ * Set the angle of a RS_PHOTO
+ * @param photo A RS_PHOTO
+ * @param angle The new angle
+ * @param relative If set to TRUE, angle will be relative to existing angle
+ */
+extern void
+rs_photo_set_angle(RS_PHOTO *photo, gdouble angle, gboolean relative)
+{
+	if (!photo) return;
+
+	if (relative)
+		photo->angle += angle;
+	else
+		photo->angle = angle;
+}
+
+/**
+ * Get the angle of a RS_PHOTO
+ * @param photo A RS_PHOTO
+ * @return The current angle
+ */
+extern gdouble
+rs_photo_get_angle(RS_PHOTO *photo)
+{
+	if (!photo) return 0.0;
+
+	return photo->angle;
+}
+
 /* Macro to create functions for changing single parameters */
 #define RS_PHOTO_SET_GDOUBLE_VALUE(setting) \
 void \
@@ -520,6 +550,81 @@ rs_photo_set_wb_from_camera(RS_PHOTO *photo, const gint snapshot)
 	return ret;
 }
 
+/**
+ * Loads a photo in to a RS_PHOTO including metadata
+ * @param filename The filename to load
+ * @param half_size Open in half size - without NN-demosaic
+ * @return A RS_PHOTO on success, NULL on error
+ */
+RS_PHOTO *
+rs_photo_load_from_file(const gchar *filename, gboolean half_size)
+{
+	RS_PHOTO *photo = NULL;
+	RS_FILETYPE *filetype;
+	RS_IMAGE16 *image;
+	guint mask;
+	gint i;
+
+	/* Try preloaded first! */
+	photo = rs_get_preloaded(filename);
+	if (photo)
+		return photo;
+
+	filetype = rs_filetype_get(filename, TRUE);
+
+	if (filetype && filetype->load)
+	{
+		image = filetype->load(filename, half_size);
+
+		if (image)
+		{
+			photo = rs_photo_new();
+
+			/* Set filename */
+			photo->filename = g_strdup(filename);
+
+			/* Set input image */
+			photo->input = image;
+
+			/* Load metadata */
+			if (rs_metadata_load(filename, photo->metadata))
+			{
+				/* Rotate photo inplace */
+				switch (photo->metadata->orientation)
+				{
+					case 90: ORIENTATION_90(photo->orientation);
+						break;
+					case 180: ORIENTATION_180(photo->orientation);
+						break;
+					case 270: ORIENTATION_270(photo->orientation);
+						break;
+				}
+			}
+
+			/* Load cache */
+			mask = rs_cache_load(photo);
+			/* If we have no cache, try to set some sensible defaults */
+			for (i=0;i<3;i++)
+			{
+				/* White balance */
+				if (!(mask & MASK_WB))
+					if (!rs_photo_set_wb_from_camera(photo, i))
+						rs_photo_set_wb_auto(photo, i);
+
+				/* Contrast */
+				if (!(mask & MASK_CONTRAST) && (photo->metadata->contrast != -1.0))
+					rs_photo_set_contrast(photo, i, photo->metadata->contrast);
+
+				/* Saturation */
+				if (!(mask & MASK_SATURATION) && (photo->metadata->saturation != -1.0))
+					rs_photo_set_saturation(photo, i, photo->metadata->saturation);
+			}
+		}
+	}
+
+	return photo;
+}
+
 /**
  * Closes a RS_PHOTO - this basically means saving cache
  * @param photo A RS_PHOTO
@@ -529,253 +634,5 @@ rs_photo_close(RS_PHOTO *photo)
 {
 	if (!photo) return;
 
-	rs_cache_save(photo);
-}
-
-/**
- * Open a photo using the dcraw-engine
- * @param filename The filename to open
- * @param half_size Open in half size - without NN-demosaic
- * @return The newly created RS_PHOTO or NULL on error
- */
-RS_PHOTO *
-rs_photo_open_dcraw(const gchar *filename, gboolean half_size)
-{
-	dcraw_data *raw;
-	RS_PHOTO *photo=NULL;
-
-	raw = (dcraw_data *) g_malloc(sizeof(dcraw_data));
-	if (!dcraw_open(raw, (char *) filename))
-	{
-		dcraw_load_raw(raw);
-		photo = rs_photo_new(NULL);
-
-		if (half_size)
-		{
-			photo->input = rs_image16_new(raw->raw.width, raw->raw.height, raw->raw.colors, 4);
-			rs_photo_open_dcraw_apply_black_and_shift_half_size(raw, photo);
-		}
-		else
-		{
-			photo->input = rs_image16_new(raw->raw.width*2, raw->raw.height*2, raw->raw.colors, 4);
-			rs_photo_open_dcraw_apply_black_and_shift(raw, photo);
-		}
-
-		photo->input->filters = raw->filters;
-		photo->input->fourColorFilters = raw->fourColorFilters;
-		photo->filename = g_strdup(filename);
-		dcraw_close(raw);
-	}
-	g_free(raw);
-	return(photo);
-}
-
-void
-rs_photo_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_PHOTO *photo)
-{
-	gushort *dst, *src;
-	gint row, col;
-	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
-
-	for(row=0;row<(raw->raw.height);row++)
-	{
-		src = (gushort *) raw->raw.image + row * raw->raw.width * 4;
-		dst = GET_PIXEL(photo->input, 0, row);
-		col = raw->raw.width;
-		while(col--)
-		{
-			register gint r, g, b, g2;
-			r  = *src++ - raw->black;
-			g  = *src++ - raw->black;
-			b  = *src++ - raw->black;
-			g2 = *src++ - raw->black;
-			r  = MAX(0, r);
-			g  = MAX(0, g);
-			b  = MAX(0, b);
-			g2 = MAX(0, g2);
-			*dst++ = (gushort)( r<<shift);
-			*dst++ = (gushort)( g<<shift);
-			*dst++ = (gushort)( b<<shift);
-			*dst++ = (gushort)(g2<<shift);
-		}
-	}
-}
-
-/* Function pointer. Initiliazed by arch binder */
-void
-(*rs_photo_open_dcraw_apply_black_and_shift)(dcraw_data *raw, RS_PHOTO *photo);
-
-void
-rs_photo_open_dcraw_apply_black_and_shift_c(dcraw_data *raw, RS_PHOTO *photo)
-{
-	gushort *dst1, *dst2, *src;
-	gint row, col;
-	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
-
-	for(row=0;row<(raw->raw.height*2);row+=2)
-	{
-		src = (gushort *) raw->raw.image + row/2 * raw->raw.width * 4;
-		dst1 = GET_PIXEL(photo->input, 0, row);
-		dst2 = GET_PIXEL(photo->input, 0, row+1);
-		col = raw->raw.width;
-		while(col--)
-		{
-			register gint r, g, b, g2;
-			r  = *src++ - raw->black;
-			g  = *src++ - raw->black;
-			b  = *src++ - raw->black;
-			g2 = *src++ - raw->black;
-			r  = MAX(0, r);
-			g  = MAX(0, g);
-			b  = MAX(0, b);
-			g2 = MAX(0, g2);
-			*dst1++ = (gushort)( r<<shift);
-			*dst1++ = (gushort)( g<<shift);
-			*dst1++ = (gushort)( b<<shift);
-			*dst1++ = (gushort)(g2<<shift);
-			*dst1++ = (gushort)( r<<shift);
-			*dst1++ = (gushort)( g<<shift);
-			*dst1++ = (gushort)( b<<shift);
-			*dst1++ = (gushort)(g2<<shift);
-			*dst2++ = (gushort)( r<<shift);
-			*dst2++ = (gushort)( g<<shift);
-			*dst2++ = (gushort)( b<<shift);
-			*dst2++ = (gushort)(g2<<shift);
-			*dst2++ = (gushort)( r<<shift);
-			*dst2++ = (gushort)( g<<shift);
-			*dst2++ = (gushort)( b<<shift);
-			*dst2++ = (gushort)(g2<<shift);
-		}
-	}
-}
-
-#if defined (__i386__) || defined (__x86_64__)
-void
-rs_photo_open_dcraw_apply_black_and_shift_mmx(dcraw_data *raw, RS_PHOTO *photo)
-{
-	char b[8];
-	volatile gushort *sub = (gushort *) b;
-	void *srcoffset;
-	void *destoffset;
-	guint x;
-	guint y;
-	gushort *src = (gushort*)raw->raw.image;
-	volatile gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
-
-	sub[0] = raw->black;
-	sub[1] = raw->black;
-	sub[2] = raw->black;
-	sub[3] = raw->black;
-
-	for (y=0; y<(raw->raw.height*2); y++)
-	{
-		destoffset = (void*) (photo->input->pixels + y*photo->input->rowstride);
-		srcoffset = (void*) (src + y/2 * raw->raw.width * photo->input->pixelsize);
-		x = raw->raw.width;
-		asm volatile (
-			"mov %3, %%"REG_a"\n\t" /* copy x to %eax */
-			"movq (%2), %%mm7\n\t" /* put black in %mm7 */
-			"movq (%4), %%mm6\n\t" /* put shift in %mm6 */
-			".p2align 4,,15\n"
-			"load_raw_inner_loop:\n\t"
-			"movq (%1), %%mm0\n\t" /* load source */
-			"movq 8(%1), %%mm1\n\t"
-			"movq 16(%1), %%mm2\n\t"
-			"movq 24(%1), %%mm3\n\t"
-			"psubusw %%mm7, %%mm0\n\t" /* subtract black */
-			"psubusw %%mm7, %%mm1\n\t"
-			"psubusw %%mm7, %%mm2\n\t"
-			"psubusw %%mm7, %%mm3\n\t"
-			"psllw %%mm6, %%mm0\n\t" /* bitshift */
-			"psllw %%mm6, %%mm1\n\t"
-			"psllw %%mm6, %%mm2\n\t"
-			"psllw %%mm6, %%mm3\n\t"
-			"movq %%mm0, (%0)\n\t" /* write destination (twice) */
-			"movq %%mm0, 8(%0)\n\t"
-			"movq %%mm1, 16(%0)\n\t"
-			"movq %%mm1, 24(%0)\n\t"
-			"movq %%mm2, 32(%0)\n\t"
-			"movq %%mm2, 40(%0)\n\t"
-			"movq %%mm3, 48(%0)\n\t"
-			"movq %%mm3, 56(%0)\n\t"
-			"sub $4, %%"REG_a"\n\t"
-			"add $64, %0\n\t"
-			"add $32, %1\n\t"
-			"cmp $3, %%"REG_a"\n\t"
-			"jg load_raw_inner_loop\n\t"
-			"jz load_raw_inner_done\n\t"
-			".p2align 4,,15\n"
-			"load_raw_leftover:\n\t"
-			"movq (%1), %%mm0\n\t" /* leftover pixels */
-			"psubusw %%mm7, %%mm0\n\t"
-			"psllw %%mm6, %%mm0\n\t"
-			"movq %%mm0, (%0)\n\t"
-			"movq %%mm0, 8(%0)\n\t"
-			"add $16, %0\n\t"
-			"add $8, %1\n\t"
-			"dec %%"REG_a"\n\t"
-			"cmp $0, %%"REG_a"\n\t"
-			"jg load_raw_leftover\n\t"
-			"load_raw_inner_done:\n\t"
-			"emms\n\t" /* clean up */
-			: "+r" (destoffset), "+r" (srcoffset)
-			: "r" (sub), "r" ((gulong)x), "r" (&shift)
-			: "%"REG_a
-			);
-	}
-}
-#endif
-
-/**
- * Open a photo using the GDK-engine
- * @param filename The filename to open
- * @param half_size Does nothing
- * @return The newly created RS_PHOTO or NULL on error
- */
-RS_PHOTO *
-rs_photo_open_gdk(const gchar *filename, gboolean half_size)
-{
-	RS_PHOTO *photo=NULL;
-	GdkPixbuf *pixbuf;
-	guchar *pixels;
-	gint rowstride;
-	gint width, height;
-	gint row,col,n,res, src, dest;
-	gdouble nd;
-	gushort gammatable[256];
-	gint alpha=0;
-	if ((pixbuf = gdk_pixbuf_new_from_file(filename, NULL)))
-	{
-		photo = rs_photo_new();
-		for(n=0;n<256;n++)
-		{
-			nd = ((gdouble) n) / 255.0;
-			res = (gint) (pow(nd, GAMMA) * 65535.0);
-			_CLAMP65535(res);
-			gammatable[n] = res;
-		}
-		rowstride = gdk_pixbuf_get_rowstride(pixbuf);
-		pixels = gdk_pixbuf_get_pixels(pixbuf);
-		width = gdk_pixbuf_get_width(pixbuf);
-		height = gdk_pixbuf_get_height(pixbuf);
-		if (gdk_pixbuf_get_has_alpha(pixbuf))
-			alpha = 1;
-		photo->input = rs_image16_new(width, height, 3, 4);
-		for(row=0;row<photo->input->h;row++)
-		{
-			dest = row * photo->input->rowstride;
-			src = row * rowstride;
-			for(col=0;col<photo->input->w;col++)
-			{
-				photo->input->pixels[dest++] = gammatable[pixels[src++]];
-				photo->input->pixels[dest++] = gammatable[pixels[src++]];
-				photo->input->pixels[dest++] = gammatable[pixels[src++]];
-				photo->input->pixels[dest++] = gammatable[pixels[src-2]];
-				src+=alpha;
-			}
-		}
-		g_object_unref(pixbuf);
-		photo->filename = g_strdup(filename);
-	}
-	return(photo);
+	rs_cache_save(photo, MASK_ALL);
 }
