@@ -137,7 +137,7 @@ struct _RSPreviewWidget
 	RS_IMAGE16 *scaled[MAX_VIEWS];
 	RS_IMAGE16 *sharpened[MAX_VIEWS];
 	GdkPixbuf *buffer[MAX_VIEWS];
-	RS_COLOR_TRANSFORM *rct[MAX_VIEWS];
+	RSColorTransform *rct[MAX_VIEWS];
 	gint dirty[MAX_VIEWS]; /* Dirty flag, used for multiple things */
 };
 
@@ -432,6 +432,13 @@ rs_preview_widget_set_cms(RSPreviewWidget *preview, void *transform)
 	{
 		DIRTY(preview->dirty[view], BUFFER);
 		rs_color_transform_set_cms_transform(preview->rct[view], transform);
+
+		/* FIXME: Hack to make sure that the Adobe-matrix gets dis-/enabled
+		 * on change, RSColorTransform should do this by itself! */
+		if (preview->photo)
+			rs_color_transform_set_from_settings(preview->rct[view],
+				preview->photo->settings[preview->snapshot[view]],
+				MASK_EXPOSURE);
 	}
 	rs_preview_widget_update(preview, FALSE);
 }
@@ -583,6 +590,9 @@ rs_preview_widget_update(RSPreviewWidget *preview, gboolean full_redraw)
 	if (!preview->photo)
 		return;
 
+	if (!GTK_WIDGET_DRAWABLE(GTK_WIDGET(preview)))
+		return;
+
 	for(view=0;view<preview->views;view++)
 	{
 		if (ISDIRTY(preview->dirty[view], SCALE))
@@ -1035,7 +1045,7 @@ buffer(RSPreviewWidget *preview, const gint view, GdkRectangle *dirty)
 	if (gdk_rectangle_intersect(&image, dirty, &clip))
 	{
 		if ((clip.width>0) && (clip.height>0))
-			preview->rct[view]->transform(preview->rct[view],
+			rs_color_transform_transform(preview->rct[view],
 				clip.width, clip.height,
 				GET_PIXEL(source, clip.x, clip.y), source->rowstride,
 				GET_PIXBUF_PIXEL(preview->buffer[view], clip.x, clip.y), gdk_pixbuf_get_rowstride(preview->buffer[view]));
@@ -1685,7 +1695,9 @@ motion(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
 	gdk_window_get_pointer(window, &x, &y, &mask);
 	view = get_view_from_coord(preview, x, y);
 
-	g_return_val_if_fail(VIEW_IS_VALID(view), TRUE);
+	/* Bail in silence */
+	if (!VIEW_IS_VALID(view))
+		return TRUE;
 
 	if (preview->photo)
 		inside_image = get_image_coord(preview, view, x, y, &scaled_x, &scaled_y, &real_x, &real_y, &max_w, &max_h);
@@ -2131,7 +2143,7 @@ make_cbdata(RSPreviewWidget *preview, const gint view, RS_PREVIEW_CALLBACK_DATA
 	cbdata->y = real_y;
 
 	/* Render current pixel */
-	preview->rct[view]->transform(preview->rct[view],
+	rs_color_transform_transform(preview->rct[view],
 		1, 1,
 		cbdata->pixel, preview->scaled[view]->rowstride,
 		&cbdata->pixel8, 1);
