@@ -22,6 +22,7 @@
 #include <config.h>
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
+#include <glib.h>
 #include "rawstudio.h"
 #include "conf_interface.h"
 #include "gettext.h"
@@ -82,6 +83,7 @@ G_DEFINE_TYPE (RSStore, rs_store, GTK_TYPE_HBOX);
 
 enum {
 	THUMB_ACTIVATED_SIGNAL,
+	GROUP_ACTIVATED_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -111,12 +113,21 @@ static gboolean tree_find_filename(GtkTreeModel *store, const gchar *filename, G
 static void cancel_clicked(GtkButton *button, gpointer user_data);
 GList *find_loadable(const gchar *path, gboolean load_8bit, gboolean load_recursive);
 void load_loadable(RSStore *store, GList *loadable, RS_PROGRESS *rsp);
+void cairo_draw_thumbnail(cairo_t *cr, GdkPixbuf *pixbuf, gint x, gint y, gint width, gint height, gdouble alphas);
+GdkPixbuf * store_group_update_pixbufs(GdkPixbuf *pixbuf, GdkPixbuf *pixbuf_clean);
 void store_group_select_n(GtkListStore *store, GtkTreeIter iter, guint n);
 gboolean store_iter_is_group(GtkListStore *store, GtkTreeIter *iter);
 void store_save_groups(GtkListStore *store);
 void store_load_groups(GtkListStore *store);
 void store_group_photos_by_iters(GtkListStore *store, GList *members);
 void store_group_photos_by_filenames(GtkListStore *store, GList *members);
+static GList *store_iter_list_to_filename_list(GtkListStore *store, GList *iters);
+void store_group_select_name(GtkListStore *store, const gchar *filename);
+void store_group_find_name(GtkListStore *store, const gchar *name, GtkTreeIter *iter, gint *n);
+void store_get_members(GtkListStore *store, GtkTreeIter *iter, GList **members);
+void store_get_type(GtkListStore *store, GtkTreeIter *iter, gint *type);
+void store_get_fullname(GtkListStore *store, GtkTreeIter *iter, gchar **fullname);
+void store_set_members(GtkListStore *store, GtkTreeIter *iter, GList *members);
 
 /**
  * Class initializer
@@ -139,6 +150,16 @@ rs_store_class_init(RSStoreClass *klass)
 		G_TYPE_NONE,
 		1,
 		G_TYPE_STRING);
+	signals[GROUP_ACTIVATED_SIGNAL] = g_signal_new ("group-activated",
+		G_TYPE_FROM_CLASS (klass),
+		G_SIGNAL_RUN_FIRST,
+		0,
+		NULL, 
+		NULL,                
+		g_cclosure_marshal_VOID__POINTER,
+		G_TYPE_NONE,
+		1,
+		G_TYPE_POINTER);
 	if (!icon_priority_1)
 	{
 		icon_priority_1	= gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_priority1.png", NULL);
@@ -193,7 +214,7 @@ rs_store_init(RSStore *store)
 		g_object_set (cell_renderer,
 			"follow-state", TRUE,
 			"height", 130,  
-			"width", 130, 
+			"width", -1, 
 			"yalign", 0.5,
 			"xalign", 0.5,
 			NULL);
@@ -255,7 +276,7 @@ rs_store_init(RSStore *store)
 	}
 
 	/* Load show filenames state from config */
-	rs_conf_get_boolean_with_default(CONF_SHOW_FILENAMES, &show_filenames, FALSE);
+	rs_conf_get_boolean_with_default(CONF_SHOW_FILENAMES, &show_filenames, DEFAULT_CONF_SHOW_FILENAMES);
 	rs_store_set_show_filenames(store, show_filenames);
 
 	/* Default to page 0 */
@@ -370,18 +391,42 @@ predict_preload(RSStore *store, gboolean initial)
 static void
 selection_changed(GtkIconView *iconview, gpointer data)
 {
+	RSStore *store = RS_STORE(data);
+	GtkTreeModel *model = GTK_TREE_MODEL(store->store);
+	GtkTreeIter iter;
+	gint type;
+	gchar *name;
+	GList *group_member_list;
+	GList *filename_list;
 	GList *selected = NULL;
 	gint num_selected;
 
 	/* Get list of selected icons */
-	gtk_icon_view_selected_foreach(iconview, icon_get_selected_names, &selected);
-
+	selected = rs_store_get_selected_iters(store);
 	num_selected = g_list_length(selected);
 
 	/* Emit signal if only one thumbnail is selected */
 	if (num_selected == 1)
-		g_signal_emit (G_OBJECT (data), signals[THUMB_ACTIVATED_SIGNAL], 0, g_list_nth_data(selected, 0));
+	{
+		iter = * (GtkTreeIter *) g_list_nth_data(selected, 0);
+		/* Get type of row */
+		gtk_tree_model_get(model, &iter, TYPE_COLUMN, &type, -1);
+		switch (type)
+		{
+			case RS_STORE_TYPE_GROUP:
+				gtk_tree_model_get(model, &iter, GROUP_LIST_COLUMN, &group_member_list, -1);
+				filename_list = store_iter_list_to_filename_list(store->store, group_member_list);
+				g_signal_emit(G_OBJECT(data), signals[GROUP_ACTIVATED_SIGNAL], 0, filename_list);
+				g_list_free(filename_list);
+				break;
+			default:
+				gtk_tree_model_get(GTK_TREE_MODEL(store->store), &iter, FULLNAME_COLUMN, &name, -1);
+				g_signal_emit(G_OBJECT(data), signals[THUMB_ACTIVATED_SIGNAL], 0, name);
+				break;
+		}
+	}
 
+	g_list_foreach(selected, (GFunc)g_free, NULL);
 	g_list_free(selected);
 
 	predict_preload(data, FALSE);
@@ -843,12 +888,10 @@ load_loadable(RSStore *store, GList *loadable, RS_PROGRESS *rsp)
 		{
 			if (filetype->load)
 			{
-				pixbuf = NULL;
-				if (filetype->thumb)
-					pixbuf = filetype->thumb(fullname);
+				pixbuf = rs_load_thumb(filetype, fullname);
 				if (pixbuf==NULL)
 				{
-					pixbuf = missing_thumb;
+					pixbuf = gdk_pixbuf_copy(missing_thumb);
 					g_object_ref (pixbuf);
 				}
 
@@ -1403,24 +1446,117 @@ rs_store_get_current_page(RSStore *store)
 	return gtk_notebook_get_current_page(store->notebook);
 }
 
+void
+cairo_draw_thumbnail(cairo_t *cr, GdkPixbuf *pixbuf, gint x, gint y, gint width, gint height, gdouble alpha)
+{
+	gdouble greyvalue = 1.0;
+
+	cairo_set_source_rgba(cr, greyvalue, greyvalue, greyvalue, 0.5);
+	cairo_rectangle(cr, x, y, width, height);
+	cairo_fill(cr);
+
+	gint pixbuf_height = gdk_pixbuf_get_height(pixbuf);
+	gint pixbuf_width = gdk_pixbuf_get_width(pixbuf);
+
+	GdkPixbuf *pixbuf_scaled = gdk_pixbuf_scale_simple(pixbuf, (pixbuf_width-4), (pixbuf_height-4), GDK_INTERP_HYPER);
+	gdk_cairo_set_source_pixbuf(cr, pixbuf_scaled, (x+2), (y+2));
+	g_object_unref(pixbuf_scaled);
+	cairo_paint(cr);
+
+	cairo_set_source_rgba(cr, 0.5, 0.5, 0.5, alpha);
+	cairo_rectangle(cr, x+1, y+1, width-2, height-2);
+	cairo_fill(cr);
+
+	return;
+}
+
+void
+calc_rotated_coordinats(gdouble a, gdouble b, gdouble R, gdouble *a2, gdouble *b2)
+{
+	gdouble c, A;
+
+	c = sqrt(a*a+b*b);
+	A = atan(a/b)+R;
+	*a2 = sin(A)*c;
+	*b2 = cos(A)*c;
+}
+
 GdkPixbuf *
 store_group_update_pixbufs(GdkPixbuf *pixbuf, GdkPixbuf *pixbuf_clean)
 {
-	gint width, height, new_width, new_height;
-	guint rowstride;
-	guchar *pixels;
-	gint channels;
-	GdkPixbuf *new_pixbuf, *pixbuf_scaled;
+	gint width, height;
+	GdkPixbuf *new_pixbuf;
 
 	width = gdk_pixbuf_get_width(pixbuf_clean);
 	height = gdk_pixbuf_get_height(pixbuf_clean);
-	
+
 	new_pixbuf = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pixbuf_clean),
 								TRUE,
 								gdk_pixbuf_get_bits_per_sample(pixbuf_clean),
 								width,
 								height);
 
+#if GTK_CHECK_VERSION(2,8,0) && defined(EXPERIMENTAL)	
+
+	gdouble a2, b2, scale, bb_x1, bb_x2, bb_y1, bb_y2, bb_height, bb_width, xoffset, yoffset, border = 1;
+
+	/* We have a bit more room with landscape-mode photos than with portrait-mode*/
+	if (height > width)
+		scale = 0.9;
+	else
+		scale = 1.0;
+
+	/* upper left of left rotation - we need a2 */
+	calc_rotated_coordinats((0-(width/2)), 128, -0.1, &a2, &b2);
+	bb_x1 = a2;
+
+	/* upper left of right rotation - we need b2 */
+	calc_rotated_coordinats((0-(width/2)), 128, 0.2, &a2, &b2);
+	bb_y1 = b2;
+
+	/* upper right of right rotation - we need b2 */
+	calc_rotated_coordinats((width/2), 128, 0.2, &a2, &b2);
+	bb_x2 = a2;
+
+	/* lower right of right rotation - we need b2 */
+	calc_rotated_coordinats((width/2),(128-height), 0.2, &a2, &b2);
+	bb_y2 = b2;
+
+	/* Calculate the magic numbers - it will work from scale 0.7 and up */
+	bb_height = ((bb_y1-bb_y2)+border*2)*scale;
+	bb_width = ((bb_x1*-1+bb_x2+10)+border*2)*scale;
+	xoffset = (bb_x2+bb_x1)/2/scale*-1;
+	yoffset = (128-(bb_y1-(128)))/scale*-1;
+
+	cairo_surface_t *surface;
+	cairo_t *cr;
+
+	surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, bb_width, bb_height);
+
+	cr = cairo_create(surface);
+
+	cairo_translate(cr, (bb_width/2), 128);
+	cairo_scale(cr, scale, scale);
+
+	cairo_rotate(cr, -0.1);
+	cairo_draw_thumbnail(cr, pixbuf_clean, (width/-2)+xoffset, yoffset, width, height, 0.7);
+	cairo_rotate(cr, 0.2);
+	cairo_draw_thumbnail(cr, pixbuf_clean, (width/-2)+xoffset, yoffset, width, height, 0.7);
+	cairo_rotate(cr, 0.1);
+	cairo_draw_thumbnail(cr, pixbuf_clean, (width/-2)+xoffset, yoffset, width, height, 0.7);
+	cairo_rotate(cr, -0.2);
+	cairo_draw_thumbnail(cr, pixbuf_clean, (width/-2)+xoffset, yoffset, width, height, 0.0);
+
+	cairo_destroy(cr);
+	new_pixbuf = cairo_convert_to_pixbuf(surface);
+#else
+
+	guint rowstride;
+	guchar *pixels;
+	gint channels;
+	gint new_width, new_height;
+	GdkPixbuf *pixbuf_scaled;
+
 	width -= 6;
 	height -= 6;
 	
@@ -1457,6 +1593,9 @@ store_group_update_pixbufs(GdkPixbuf *pixbuf, GdkPixbuf *pixbuf_clean)
 	gdk_pixbuf_copy_area(pixbuf_scaled,
 						 0, 0, width, height,
 						 new_pixbuf, 1, 1);
+
+#endif
+
 	return new_pixbuf;
 }
 
@@ -1472,9 +1611,8 @@ store_group_select_n(GtkListStore *store, GtkTreeIter iter, guint n)
 	guint priority;
 	gboolean exported;
 
-	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
-					   GROUP_LIST_COLUMN, &members,
-					   -1);
+	store_get_members(store, &iter, &members);
+
 	child_iter = (GtkTreeIter *) g_list_nth_data(members, n);
 
 	gtk_tree_model_get(GTK_TREE_MODEL(store), child_iter,
@@ -1532,10 +1670,7 @@ rs_store_group_photos(RSStore *store)
 			s = selected->data;
 			if (store_iter_is_group(store->store, s))
 			{
-				gtk_tree_model_get(GTK_TREE_MODEL(store->store), s,
-								   GROUP_LIST_COLUMN, &members,
-								   -1);
-
+				store_get_members(store->store, s, &members);
 				newmembers = g_list_concat(newmembers, members);
 
 				if (group)
@@ -1584,9 +1719,8 @@ rs_store_ungroup_photos(RSStore *store)
 		if (store_iter_is_group(store->store, g_list_nth_data(selected, n)))
 		{
 			iter = (GtkTreeIter *) g_list_nth_data(selected, n);
-			gtk_tree_model_get(GTK_TREE_MODEL(store->store), iter,
-							   GROUP_LIST_COLUMN, &members,
-							   -1);
+			store_get_members(store->store, iter, &members);
+
 			for( m = 0; m < g_list_length(members); m++)
 			{
 				child_iter = (GtkTreeIter *) g_list_nth_data(members, m);
@@ -1603,9 +1737,9 @@ rs_store_ungroup_photos(RSStore *store)
 gboolean
 store_iter_is_group(GtkListStore *store, GtkTreeIter *iter)
 {
-	guint t;
+	gint t;
 
-	gtk_tree_model_get (GTK_TREE_MODEL(store), iter, TYPE_COLUMN, &t, -1);
+	store_get_type(store, iter, &t);
 	if (t == RS_STORE_TYPE_GROUP)
 		return TRUE;
 	else
@@ -1632,7 +1766,7 @@ store_save_groups(GtkListStore *store) {
 	xmlTextWriterPtr writer;
 
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-	gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, FULLNAME_COLUMN, &filename, -1);
+	store_get_fullname(store, &iter, &filename);
 
 	dotdir = rs_dotdir_get(filename);
 	GString *gs = g_string_new(dotdir);
@@ -1653,25 +1787,24 @@ store_save_groups(GtkListStore *store) {
 		{
 			if (store_iter_is_group(store, &iter))
 			{
-				gchar *selected;
-				GList *members;
+				gchar *selected = NULL;
+				GList *members, *filenames;
 
 				xmlTextWriterStartElement(writer, BAD_CAST "group");
 
 				// Find selected member and place this first in XML
-				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
-					   FULLNAME_COLUMN, &selected,
-					   -1);
+				store_get_fullname(store, &iter, &selected);
+
 				xmlTextWriterWriteFormatElement(writer, BAD_CAST "member", "%s", selected);
 
-				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
-					   GROUP_LIST_COLUMN, &members,
-					   -1);
+				store_get_members(store, &iter, &members);
+
 				gint m;
-				for( m = 0; m < g_list_length(members); m++)
+				filenames = store_iter_list_to_filename_list(store, members);
+
+				for( m = 0; m < g_list_length(filenames); m++)
 				{
-					GtkTreeIter *child_iter = (GtkTreeIter *) g_list_nth_data(members, m);
-					gtk_tree_model_get (GTK_TREE_MODEL(store), child_iter, FULLNAME_COLUMN, &filename, -1);
+					filename = g_list_nth_data(filenames, m);
 					if (!g_str_equal(selected, filename))
 						xmlTextWriterWriteFormatElement(writer, BAD_CAST "member", "%s", filename);
 				}
@@ -1697,7 +1830,12 @@ store_load_groups(GtkListStore *store) {
 	g_assert(store != NULL);
 
 	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-	gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, FULLNAME_COLUMN, &filename, -1);
+
+	/* Don't try opening groups when there is no loaded photos in the store. */
+	  if(!gtk_list_store_iter_is_valid(store, &iter))
+		return;
+
+	store_get_fullname(store, &iter, &filename);
 	dotdir = rs_dotdir_get(filename);
 
 	GString *gs = g_string_new(dotdir);
@@ -1795,3 +1933,191 @@ GList
 {
 	return g_list_sort(selected, (GCompareFunc) g_utf8_collate);
 }
+
+void
+rs_store_auto_group(RSStore *store)
+{
+	RS_FILETYPE *filetype = NULL;
+	gchar *filename = NULL;
+	gint timestamp = 0, timestamp_old = 0;
+	gint exposure;
+	RS_METADATA *meta;
+	GList *filenames = NULL;
+	GtkTreeIter iter;
+
+	// TODO: remove all existing groups in iconview.
+	
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store->store), &iter);
+	do
+	{
+		store_get_fullname(GTK_LIST_STORE(store->store), &iter, &filename);
+		filetype = rs_filetype_get(filename, TRUE);
+		if (filetype)
+		{
+			if(filetype->load_meta)
+			{
+				meta = rs_metadata_new();
+				filetype->load_meta(filename, meta);
+				
+				if (!meta->timestamp)
+					return;
+				
+				timestamp = meta->timestamp;
+				exposure = (1/meta->shutterspeed);
+
+				if (timestamp > timestamp_old + 1)
+				{
+					if (g_list_length(filenames) > 1)
+						store_group_photos_by_filenames(store->store, filenames);
+					g_list_free(filenames);
+					filenames = NULL;
+				}
+				timestamp_old = timestamp + exposure;
+				g_free(meta);
+			}
+		}
+		filenames = g_list_append(filenames, filename);
+	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store->store), &iter));
+
+	store_group_photos_by_filenames(store->store, filenames);
+	g_list_free(filenames);
+	filenames = NULL;
+}
+
+static GList *
+store_iter_list_to_filename_list(GtkListStore *store, GList *iters)
+{
+	gint n;
+	gchar *filename = NULL;
+	GList *filenames = NULL;
+
+	for (n=0; n<g_list_length(iters); n++)
+	{
+		GtkTreeIter *iter = (GtkTreeIter *) g_list_nth_data(iters, n);
+		store_get_fullname(store, iter, &filename);
+		filenames = g_list_append(filenames, filename);
+	}
+
+	return filenames;
+}
+
+void
+store_group_select_name(GtkListStore *store, const gchar *filename)
+{
+	GtkTreeIter iter;
+	gint n = -1;
+
+	store_group_find_name(store, filename, &iter, &n);
+	store_group_select_n(store, iter, n);
+}
+
+void 
+store_group_find_name(GtkListStore *store, const gchar *name, GtkTreeIter *iter, gint *n)
+{
+	gint type;
+	gint i;
+	GList *members = NULL;
+	GList *filenames = NULL;
+
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), iter);
+	do
+	{
+		store_get_type(store, iter, &type);
+
+		if (type == RS_STORE_TYPE_GROUP)
+		{
+			store_get_members(store, iter, &members);
+			filenames = store_iter_list_to_filename_list(store, members);
+
+			for(i = 0; i < g_list_length(filenames); i++)
+			{
+				if (g_str_equal(g_list_nth_data(filenames, i), name))
+				{
+					*n = i;
+					return;
+				}
+			}
+			g_list_free(filenames);
+		}
+	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store), iter));
+
+	*n = -1;
+	return;
+}
+
+void
+rs_store_group_select_name(RSStore *store, const gchar *filename)
+{
+	store_group_select_name(store->store, filename);
+}
+
+void 
+store_group_ungroup_name(GtkListStore *store, const gchar *name)
+{
+	GtkTreeIter iter, *child_iter;
+	gint n = -1;
+	GList *members;
+
+	store_group_find_name(store, name, &iter, &n);
+	
+	gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 
+						GROUP_LIST_COLUMN, &members,
+						-1);
+	
+	child_iter = (GtkTreeIter *) g_list_nth_data(members, n);
+	gtk_list_store_set(store, child_iter,
+						TYPE_COLUMN, RS_STORE_TYPE_FILE,
+					   -1);
+
+	members = g_list_remove(members, g_list_nth_data(members,n));
+
+	/* If group now only has one member, we destroy the group */
+	if (g_list_length(members) == 1)
+	{
+		child_iter = (GtkTreeIter *) g_list_nth_data(members, 0);
+		gtk_list_store_set(store, child_iter,
+							TYPE_COLUMN, RS_STORE_TYPE_FILE,
+						   -1);
+		gtk_list_store_remove (store, &iter);
+	}
+	store_set_members(store, &iter, members);
+	store_save_groups(store);
+}
+
+void
+rs_store_group_ungroup_name(RSStore *store, const gchar *filename)
+{
+	store_group_ungroup_name(store->store, filename);
+}
+
+void
+store_get_members(GtkListStore *store, GtkTreeIter *iter, GList **members)
+{
+	gtk_tree_model_get (GTK_TREE_MODEL(store), iter,
+						GROUP_LIST_COLUMN, members,
+						-1);
+}
+
+void
+store_get_type(GtkListStore *store, GtkTreeIter *iter, gint *type)
+{
+	gtk_tree_model_get (GTK_TREE_MODEL(store), iter,
+						TYPE_COLUMN, type,
+						-1);
+}
+
+void
+store_get_fullname(GtkListStore *store, GtkTreeIter *iter, gchar **fullname)
+{
+	gtk_tree_model_get (GTK_TREE_MODEL(store), iter, 
+						FULLNAME_COLUMN, fullname,
+						-1);
+}
+
+void
+store_set_members(GtkListStore *store, GtkTreeIter *iter, GList *members)
+{
+	gtk_list_store_set (store, iter, 
+						GROUP_LIST_COLUMN, members,
+						-1);
+}
